//
//  Backend.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "Backend.h"


@implementation Backend

- (id)init {
	if (self = [super init]) {
		data = [[NSMutableDictionary alloc] initWithCapacity:3];
	}
	return self;
}

- (void)setAddress:(NSString *)address {
	if (address) {
		[data setObject:address forKey:@"Address"];
	}
	else {
		[data setObject:[NSNull null] forKey:@"Address"];
	}
}

- (NSString *)address {
	id address = [data objectForKey:@"Address"];
	if (address != [NSNull null]) {
		return address;
	}
	else {
		return nil;
	}
}

- (void)setHostname:(NSString *)hostname {
	if (hostname) {
		[data setObject:hostname forKey:@"Hostname"];
	}
	else {
		[data setObject:[NSNull null] forKey:@"Hostname"];
	}
}

- (NSString *)hostname {
	id hostname = [data objectForKey:@"Hostname"];
	if (hostname != [NSNull null]) {
		return hostname;
	}
	else {
		return nil;
	}
}

- (void)setPort:(unsigned short)port {
	[data setObject:[NSNumber numberWithUnsignedShort:port] forKey:@"Port"];
}

- (unsigned short)port {
	return [[data objectForKey:@"Port"] unsignedShortValue];
}

- (NSUInteger)hash {
	return [data hash];
}

- (NSMutableDictionary *)data {
	return data;
}

- (BOOL)isEqual:(id)anObject {
	Backend *backend = anObject;
	return [data isEqual:[backend data]];
}

- (void)dealloc {
	[data release];
	[super dealloc];
}

+ (Backend *)backendWithAddress:(NSString *)address hostname:(NSString *)hostname port:(unsigned short)port {
	Backend *backend = [[[Backend alloc] init] autorelease];
	backend.address = address;
	backend.hostname = hostname;
	backend.port = port;
	return backend;
}

// NSCoding

- (id)initWithCoder:(NSCoder *)decoder {
	if (self = [super init]) {
		data = [[NSMutableDictionary alloc] initWithCoder:decoder];
	}
	return self;
}

- (void)encodeWithCoder:(NSCoder *)encoder {
	[data encodeWithCoder:encoder];
}

@end
