//
//  BackendLocator.h
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import <dns_sd.h>
#import <CFNetwork/CFNetwork.h>
#import <UIKit/UIKit.h>

@class Backend;

@protocol BackendLocatorDelegate

- (void)foundBackend:(Backend *)backend;

@end

@interface BackendLocator : NSObject {
	id <NSObject, BackendLocatorDelegate> delegate;
	CFSocketRef socket;
	NSData *addrData;
	CFRunLoopSourceRef runLoopSource;
	NSMutableArray *mDNSRequests;
}

@property (retain) id <NSObject, BackendLocatorDelegate> delegate;
@property (readonly) NSMutableArray *mDNSRequests;

- (void)poll;
- (void)receiveData:(CFDataRef)data;
- (void)resolvedHost:(CFHostRef)host port:(NSString *)stringPort;
- (void)mDNSReverseResolutionWithName:(const char *)name data:(const char *)data length:(uint16_t)rdlen port:(unsigned short)stringPort error:(DNSServiceErrorType)error service:(DNSServiceRef)service;

@end

@interface mDNSRequest : NSObject
{
	DNSServiceRef service;
	CFSocketRef socket;
	CFRunLoopSourceRef runLoopSource;
}

@property (assign) DNSServiceRef service;
@property (assign) CFSocketRef socket;
@property (assign) CFRunLoopSourceRef runLoopSource;

@end
