//
//  BackendSettings.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "BackendSettings.h"

@implementation BackendSettings

- (id)initWithBackendAddress:(NSString *)newBackend {
	if (self = [super initWithStyle:UITableViewStyleGrouped]) {
		self.title = @"Backend Settings";
		
		backend = [newBackend retain];
		
		manual = [[UISwitch alloc] init];
		manual.on = ([[DEFAULTS objectForKey:DATABASE_ADDRESS_KEY] objectForKey:backend] != nil);
		[manual addTarget:self action:@selector(setManual) forControlEvents:UIControlEventValueChanged];
		
		self.navigationItem.hidesBackButton = YES;
		self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(done)];
	}
	return self;
}

- (void)setManual {
	[self.tableView reloadData];
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return 2;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	switch (section) {
		case 0:
			return 1;
			
		case 1:
			if (manual.on) {
				return 2;
			}
			else {
				return 1;
			}
	}
	
	return 0;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	UITableViewCell *cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:nil] autorelease];
	
	switch (indexPath.section) {
		case 0:
			if (!pin) {
				pin = [[UITextField alloc] initWithFrame:CGRectMake(232, 7, 58, 29)];
				pin.placeholder = @"1234";
				pin.secureTextEntry = YES;
				pin.borderStyle = UITextBorderStyleRoundedRect;
				pin.delegate = self;
			}
			if (pin.superview) {
				[pin removeFromSuperview];
			}
			pin.text = [[DEFAULTS objectForKey:BACKEND_PIN_KEY] objectForKey:backend];
			[cell.contentView addSubview:pin];
			
			cell.textLabel.text = @"Security Pin:";
			cell.textLabel.backgroundColor = [UIColor clearColor];
			
			break;
			
		case 1:
			if (indexPath.row == 0) {
				cell.textLabel.text = @"Override Address:";
				if (manual.superview) {
					[manual removeFromSuperview];
				}
				cell.accessoryView = manual;
			}
			else {
				if (!address) {
					address = [[UITextField alloc] init];
					address.frame = CGRectMake(90, 10, 200, 20);
					address.text = [[DEFAULTS objectForKey:DATABASE_ADDRESS_KEY] objectForKey:backend];
					address.placeholder = @"Hostname Or IP Address";
					address.textAlignment = UITextAlignmentRight;
					address.autocorrectionType = UITextAutocorrectionTypeNo;
					address.keyboardType = UIKeyboardTypeURL;
				}
				if (address.superview) {
					[address removeFromSuperview];
				}
				[cell.contentView addSubview:address];
		
				cell.textLabel.text = @"Address:";
				cell.textLabel.backgroundColor = [UIColor clearColor];
			}
			break;
	}
			
	cell.selectionStyle = UITableViewCellSelectionStyleNone;

	return cell;
}

- (void)done {
	NSMutableDictionary *backendsDictionary = [NSMutableDictionary dictionaryWithDictionary:[DEFAULTS objectForKey:DATABASE_ADDRESS_KEY]];
	if (manual.on) {
		[backendsDictionary setObject:address.text forKey:backend];
	}
	else {
		[backendsDictionary removeObjectForKey:backend];
	}
	
	if ([backendsDictionary count]) {
		[DEFAULTS setObject:backendsDictionary forKey:DATABASE_ADDRESS_KEY];
	}
	else {
		[DEFAULTS removeObjectForKey:DATABASE_ADDRESS_KEY];
	}
	
	if (pin.text) {
		backendsDictionary = [NSMutableDictionary dictionaryWithDictionary:[DEFAULTS objectForKey:BACKEND_PIN_KEY]];
		[backendsDictionary setObject:pin.text forKey:backend];
		[DEFAULTS setObject:backendsDictionary forKey:BACKEND_PIN_KEY];
	}
	
	[self.navigationController popViewControllerAnimated:YES];
}

- (void)dealloc {
	[backend release];
	[manual release];
	[address release];
	[pin release];
	[super dealloc];
}

// UITextFieldDelegate

- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string {
	if (string.length && textField.text.length >= 4) {
		return NO;
	}
	return YES;
}

@end
