//
//  ButtonPickerController.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "ButtonPickerController.h"
#import "LIRCButtonEditor.h"
#import "RemoteButton.h"

@implementation ButtonPickerController

@synthesize pickedButton;
@synthesize targetController;

- (id)init
{
	if (self = [super init]) {
		self.title = @"Choose Button To Add";
	}
	return self;
}


- (void)loadView
{
	UIBarButtonItem *cancelItem = [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemCancel target:self action:@selector(dismiss)] autorelease];
	self.navigationItem.leftBarButtonItem = cancelItem;
	
	UIScrollView *scrollView = [[[UIScrollView alloc] initWithFrame:[UIScreen mainScreen].applicationFrame] autorelease];
	scrollView.backgroundColor = [UIColor blackColor];
	self.view = scrollView;
	
	UIView *contentView = [[[UIView alloc] init] autorelease];
	[scrollView addSubview:contentView];
	
	NSDictionary *pickerInfo = [[NSDictionary alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"ButtonPicker" ofType:@"plist"]];
	
	NSInteger contentHeight = 0;
	
	for (NSString *context in [NSArray arrayWithObjects:@"Global", @"Playback", @"Music", @"Weather", @"LIRC", nil]) {
		NSDictionary *contextInfo;
		if ([context isEqualToString:@"LIRC"]) {
			contextInfo = [[NSUserDefaults standardUserDefaults] objectForKey:@"LIRC Buttons"];
		}
		else {
			contextInfo = [pickerInfo objectForKey:context];
		}
		
		UIView *contextView = [[[UIView alloc] initWithFrame:CGRectMake(0, contentHeight, scrollView.bounds.size.width, [[contextInfo objectForKey:@"Height"] floatValue] + 50)] autorelease];
		[contentView addSubview:contextView];
		contentHeight += [[contextInfo objectForKey:@"Height"] floatValue] + 50;
		
		UILabel *title = [[[UILabel alloc] initWithFrame:CGRectMake(0, 0, contextView.bounds.size.width, 50)] autorelease];
		title.text = [contextInfo objectForKey:@"Title"];
		title.font = [UIFont boldSystemFontOfSize:20];
		title.backgroundColor = [UIColor blackColor];
		title.textColor = [UIColor whiteColor];
		title.textAlignment = UITextAlignmentCenter;
		[contextView addSubview:title];
		
		if ([context isEqualToString:@"LIRC"]) {
			UIButton *newButton = [UIButton buttonWithType:UIButtonTypeContactAdd];
			newButton.frame = CGRectMake(scrollView.bounds.size.width - 45, 13, 30, 30);
			[newButton addTarget:self action:@selector(newLIRCButton) forControlEvents:UIControlEventTouchUpInside];
			[contextView addSubview:newButton];
		}
		
		UIView *buttonsView = [[[UIView alloc] initWithFrame:CGRectMake(0, 50, contextView.bounds.size.width, [[contextInfo objectForKey:@"Height"] floatValue])] autorelease];
		[contextView addSubview:buttonsView];
		if ([context isEqualToString:@"LIRC"]) {
			LIRCButtonsView = buttonsView;
		}
		
		for (NSDictionary *buttonInfo in [contextInfo objectForKey:@"Buttons"]) {
			RemoteButton *remoteButton = [[[RemoteButton alloc] initWithInfo:buttonInfo] autorelease];
			remoteButton.buttonPickerController = self;
			[buttonsView addSubview:remoteButton];
		}
	}
	
	[pickerInfo release];
	
	contentView.frame = CGRectMake(0, 0, scrollView.bounds.size.width, contentHeight + self.navigationController.navigationBar.frame.size.height);
	scrollView.contentSize = CGSizeMake(scrollView.bounds.size.width, contentHeight + self.navigationController.navigationBar.frame.size.height + 10);
	[scrollView becomeFirstResponder];
}

- (void)newLIRCButton {
	UINavigationController *modalController = [[[UINavigationController alloc] initWithRootViewController:[[[LIRCButtonEditor alloc] initWithButton:nil pickerController:self] autorelease]] autorelease];
	modalController.navigationBar.barStyle = UIBarStyleBlackOpaque;
	[self presentModalViewController:modalController animated:YES];
}

- (void)addLIRCButton:(RemoteButton *)button {
	NSMutableDictionary *LIRCButtons = [NSMutableDictionary dictionaryWithDictionary:[[NSUserDefaults standardUserDefaults] objectForKey:@"LIRC Buttons"]];
	[LIRCButtons setObject:[NSMutableArray arrayWithArray:[LIRCButtons objectForKey:@"Buttons"]] forKey:@"Buttons"];
	
	int LIRCButtonIndex = [[LIRCButtons objectForKey:@"Buttons"] count];
	button.frame = CGRectMake(10 + 60 * (LIRCButtonIndex % 5), 15 + 60 * (LIRCButtonIndex / 5), 60, 60);	

	[button.info setObject:[NSArray arrayWithObjects:[NSNumber numberWithFloat:button.frame.origin.x + 30], [NSNumber numberWithFloat:button.frame.origin.y + 30], nil] forKey:@"Center"];
	[button.info setObject:[NSArray arrayWithObjects:[NSNumber numberWithFloat:button.frame.size.width], [NSNumber numberWithFloat:button.frame.size.height], nil] forKey:@"Size"];
	
	[[LIRCButtons objectForKey:@"Buttons"] addObject:button.info];
	
	[button setNeedsDisplay];
	[LIRCButtonsView addSubview:button];
	
	button.buttonPickerController = self;
	
	if (LIRCButtonIndex % 5 == 0) {
		UIView *contextView = LIRCButtonsView.superview;
		UIView *contentView = LIRCButtonsView.superview.superview;
		UIScrollView *scrollView = (UIScrollView *)LIRCButtonsView.superview.superview.superview;

		CGRect frame;
		frame = LIRCButtonsView.frame;
		frame.size.height += 60;
		LIRCButtonsView.frame = frame;
		frame = contextView.frame;
		frame.size.height += 60;
		contextView.frame = frame;
		frame = contentView.frame;
		frame.size.height += 60;
		contentView.frame = frame;
		
		scrollView.contentSize = CGSizeMake(scrollView.contentSize.width, scrollView.contentSize.height + 60);
		scrollView.contentOffset = CGPointMake(scrollView.contentOffset.x, scrollView.contentOffset.y + 60);
		
		[LIRCButtons setObject:[NSNumber numberWithFloat:LIRCButtonsView.frame.size.height] forKey:@"Height"];
	}

	[[NSUserDefaults standardUserDefaults] setObject:LIRCButtons forKey:@"LIRC Buttons"];
}

- (void)editLIRCButton:(RemoteButton *)button {
	UINavigationController *modalController = [[[UINavigationController alloc] initWithRootViewController:[[[LIRCButtonEditor alloc] initWithButton:button pickerController:self] autorelease]] autorelease];
	modalController.navigationBar.barStyle = UIBarStyleBlackOpaque;
	[self presentModalViewController:modalController animated:YES];
}

- (void)deleteLIRCButton:(RemoteButton *)button {
	NSMutableDictionary *LIRCButtons = [NSMutableDictionary dictionaryWithDictionary:[[NSUserDefaults standardUserDefaults] objectForKey:@"LIRC Buttons"]];
	NSMutableArray *buttonsArray = [NSMutableArray arrayWithArray:[LIRCButtons objectForKey:@"Buttons"]];
	
	NSDictionary *info;
	for (info in buttonsArray) {
		if ([[button.info objectForKey:@"Title"] isEqualToString:[info objectForKey:@"Title"]]) {
			break;
		}
	}
	[buttonsArray removeObject:info];
	
	for (RemoteButton *currentButton in LIRCButtonsView.subviews) {
		if (currentButton.center.y > button.center.y || (currentButton.center.y == button.center.y && currentButton.center.x > button.center.x)) {
			if (currentButton.center.x > 60) {
				currentButton.center = CGPointMake(currentButton.center.x - 60, currentButton.center.y);
			}
			else {
				currentButton.center = CGPointMake(280, currentButton.center.y - 60);
			}
			
			NSArray *tempArray = [NSArray arrayWithArray:buttonsArray];
			for (NSDictionary *info in tempArray) {
				if ([[info objectForKey:@"Title"] isEqualToString:[currentButton.info objectForKey:@"Title"]]) {
					NSMutableDictionary *mutableInfo = [NSMutableDictionary dictionaryWithDictionary:info];
					[mutableInfo setObject:[NSArray arrayWithObjects:[NSNumber numberWithFloat:currentButton.center.x], [NSNumber numberWithFloat:currentButton.center.y], nil] forKey:@"Center"];
					[buttonsArray replaceObjectAtIndex:[tempArray indexOfObject:info] withObject:mutableInfo];
					break;
				}
			}		
		}
	}
	
	[LIRCButtons setObject:buttonsArray forKey:@"Buttons"];
	
	[button removeFromSuperview];
	
	if ([buttonsArray count] % 5 == 0) {
		UIView *contextView = LIRCButtonsView.superview;
		UIView *contentView = LIRCButtonsView.superview.superview;
		UIScrollView *scrollView = (UIScrollView *)LIRCButtonsView.superview.superview.superview;
		
		CGRect frame;
		frame = LIRCButtonsView.frame;
		frame.size.height -= 60;
		LIRCButtonsView.frame = frame;
		frame = contextView.frame;
		frame.size.height -= 60;
		contextView.frame = frame;
		frame = contentView.frame;
		frame.size.height -= 60;
		contentView.frame = frame;
		
		scrollView.contentSize = CGSizeMake(scrollView.contentSize.width, scrollView.contentSize.height - 60);
		
		[LIRCButtons setObject:[NSNumber numberWithFloat:LIRCButtonsView.frame.size.height] forKey:@"Height"];
	}

	[[NSUserDefaults standardUserDefaults] setObject:LIRCButtons forKey:@"LIRC Buttons"];
	
}

- (void)dismiss {
	pickedButton = nil;
	[targetController dismissModalViewControllerAnimated:YES];
}

- (void)viewDidDisappear:(BOOL)animated {
	[pickedButton pickerGone];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
	return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

@end
