//
//  ChannelScrollView.m
//  MyMote
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "ChannelsView.h"
#import "ChannelScrollView.h"
#import "GuideResponse.h"
#import "GuideScrollView.h"
#import "ProgramGuideData.h"

@implementation ChannelScrollView

@synthesize guideScrollView;

- (id)initWithGuideData:(ProgramGuideData *)newData {
	if (self = [super init]) {
		self.autoresizingMask = UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleRightMargin;
		self.delegate = self;
		self.clearsContextBeforeDrawing = NO;
		self.opaque = YES;
		
		data = [newData retain];
	}
	return self;
}

- (void)dataChanged {
	CGSize contentSize = CGSizeMake(EPG_CHANNEL_WIDTH, [data.response.channels count] * EPG_ROW_HEIGHT);
	if (self.contentSize.width != contentSize.width || self.contentSize.height != contentSize.height) {
		self.contentOffset = CGPointZero;
	}
	self.contentSize = contentSize;
	
	for (ChannelsView *channelsView in self.subviews) {
		[channelsView removeFromSuperview];
	}
	
	int y;
	for (y = 0; y < contentSize.height / EPG_TILE_SIZE; y++) {
		int height = MIN(EPG_TILE_SIZE, contentSize.height - y * EPG_TILE_SIZE);
		int x;
		for (x = 0; x < contentSize.width / EPG_TILE_SIZE; x++) {
			int width = MIN(EPG_TILE_SIZE, contentSize.width - x * EPG_TILE_SIZE);
			[self addSubview:[[[ChannelsView alloc] initWithFrame:CGRectMake(x * EPG_TILE_SIZE, y * EPG_TILE_SIZE, width, height) programGuideData:data] autorelease]];
		}
	}
}

- (void)dealloc {
	[data release];
	[guideScrollView release];
	[super dealloc];
}

// UIScrollViewDelegate

- (void)scrollViewWillBeginDragging:(UIScrollView *)scrollView {
	guideScrollView.channelScrolling = YES;
}

- (void)scrollViewDidScroll:(UIScrollView *)scrollView {
	guideScrollView.contentOffset = CGPointMake(guideScrollView.contentOffset.x, scrollView.contentOffset.y);
}

- (void)scrollViewDidEndDragging:(UIScrollView *)scrollView willDecelerate:(BOOL)decelerate {
	if (!decelerate) {
		guideScrollView.channelScrolling = NO;
	}
}

- (void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView {
	guideScrollView.channelScrolling = NO;
}

@end
