//
//  ChannelsView.m
//  MyMote
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import <QuartzCore/CALayer.h>

#import "Channel.h"
#import "ChannelsView.h"
#import "GuideResponse.h"
#import "ProgramGuideData.h"

@implementation ChannelsView

- (id)initWithFrame:(CGRect)frame programGuideData:(ProgramGuideData *)data {
	if (self = [super initWithFrame:frame]) {
		self.clearsContextBeforeDrawing = NO;
		self.opaque = YES;

		int startRow = self.frame.origin.y / EPG_ROW_HEIGHT;
		int endRow = startRow + self.bounds.size.height / EPG_ROW_HEIGHT + 1;
		if (endRow >= [data.response.channels count]) {
			endRow = [data.response.channels count] - 1;
		}
		
		CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
		CGContextRef ctx = CGBitmapContextCreate(NULL, self.bounds.size.width * SCALE, self.bounds.size.height * SCALE, 8, self.bounds.size.width * SCALE * 4, colorSpace, kCGImageAlphaPremultipliedLast);
		
		CGContextScaleCTM(ctx, 1, -1);
		CGContextTranslateCTM(ctx, 0, -self.bounds.size.height * SCALE);
		
		int row;
		for (row = startRow; row <= endRow; row++) {
			[[data.response.channels objectAtIndex:row] drawInRect:CGRectMake(SCALE, (row * EPG_ROW_HEIGHT + 1 - self.frame.origin.y) * SCALE, (EPG_CHANNEL_WIDTH - 2) * SCALE, (EPG_ROW_HEIGHT - 2) * SCALE) context:ctx];
		}
		CGImageRef image = CGBitmapContextCreateImage(ctx);
		
		CGContextRelease(ctx);
		CGColorSpaceRelease(colorSpace);
		
		self.layer.contents = (id)image;
		if ([self.layer respondsToSelector:@selector(contentsScale)])
			self.layer.contentsScale = SCALE;
		CFRelease(image);
	}
	return self;
}

@end
