//
//  FrontendAddress.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "FrontendAddress.h"

@implementation FrontendAddress

- (id)initWithBackend:(NSString *)newBackend frontend:(NSString *)newFrontend {
	if (self = [super initWithStyle:UITableViewStyleGrouped]) {
		self.title = @"Frontend Address";
		
		backend = [newBackend retain];
		frontend = [newFrontend retain];
		
		manual = [[UISwitch alloc] init];
		manual.on = ([[[DEFAULTS objectForKey:FRONTEND_ADDRESS_KEY] objectForKey:backend] objectForKey:frontend] != nil);
		[manual addTarget:self action:@selector(setManual) forControlEvents:UIControlEventValueChanged];
		
		self.navigationItem.hidesBackButton = YES;
		self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(done)];
	}
	return self;
}

- (void)setManual {
	[self.tableView reloadData];
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	if (manual.on) {
		return 2;
	}
	else {
		return 1;
	}
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	UITableViewCell *cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:nil] autorelease];
	
	if (indexPath.row == 0) {
		cell.textLabel.text = @"Override Address:";
		if (manual.superview) {
			[manual removeFromSuperview];
		}
		cell.accessoryView = manual;
	}
	else {
		if (!address) {
			address = [[UITextField alloc] init];
			address.frame = CGRectMake(90, 10, 200, 20);
			address.text = [[[DEFAULTS objectForKey:FRONTEND_ADDRESS_KEY] objectForKey:backend] objectForKey:frontend];
			address.placeholder = @"Hostname Or IP Address";
			address.textAlignment = UITextAlignmentRight;
			address.autocorrectionType = UITextAutocorrectionTypeNo;
			address.keyboardType = UIKeyboardTypeURL;
		}
		if (address.superview) {
			[address removeFromSuperview];
		}
		[cell.contentView addSubview:address];
		
		cell.textLabel.text = @"Address:";
		cell.textLabel.backgroundColor = [UIColor clearColor];
	}
	
	cell.selectionStyle = UITableViewCellSelectionStyleNone;

	return cell;
}

- (void)done {
	NSMutableDictionary *frontendsDictionary = [NSMutableDictionary dictionaryWithDictionary:[[DEFAULTS objectForKey:FRONTEND_ADDRESS_KEY] objectForKey:backend]];
	if (manual.on) {
		[frontendsDictionary setObject:address.text forKey:frontend];
	}
	else {
		[frontendsDictionary removeObjectForKey:frontend];
	}
	
	NSMutableDictionary *backendsDictionary = [NSMutableDictionary dictionaryWithDictionary:[DEFAULTS objectForKey:FRONTEND_ADDRESS_KEY]];
	if ([frontendsDictionary count]) {
		[backendsDictionary setObject:frontendsDictionary forKey:backend];
	}
	else {
		[backendsDictionary removeObjectForKey:backend];
	}
	
	if ([backendsDictionary count]) {
		[DEFAULTS setObject:backendsDictionary forKey:FRONTEND_ADDRESS_KEY];
	}
	else {
		[DEFAULTS removeObjectForKey:FRONTEND_ADDRESS_KEY];
	}
	
	[self.navigationController popViewControllerAnimated:YES];
}

- (void)dealloc {
	[backend release];
	[frontend release];
	[manual release];
	[address release];
	[super dealloc];
}

@end
