//
//  FrontentSettings.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "Backend.h"
#import "FrontendSettings.h"

@implementation FrontendSettings

- (id)initWithBackend:(Backend *)backend frontend:(NSString *)newFrontend delegate:(id <NSObject, FrontendSettingsDelegate>)newDelegate {
	if (self = [super init]) {
		frontend = [[NSString stringWithString:newFrontend] retain];
		delegate = [newDelegate retain];
		
		settingsXML = [[NSMutableData alloc] init];
		
		connection = [[NSURLConnection connectionWithRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:[NSString stringWithFormat:@"http://%@:%hu/Myth/GetSetting?HostName=%@", (backend.hostname ? backend.hostname : backend.address), backend.port, frontend]]] delegate:self] retain];
	}
	return self;
}

// For NSURLConnection

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
	[settingsXML setLength:0];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
	[settingsXML appendData:data];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
	NSLog(@"Connection failed: %@ %@", [error localizedDescription], [[error userInfo] objectForKey:NSErrorFailingURLStringKey]);
	[delegate frontendSettings:self invalidFrontend:frontend];
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection {
	parser = [[NSXMLParser alloc] initWithData:settingsXML];
	parser.delegate	= self;
	
	if ([parser parse] == NO) {
		NSLog(@"Error Retrieving Frontend Settings: %@", [[parser parserError] localizedDescription]);
		[delegate frontendSettings:self invalidFrontend:frontend];
		return;
	}
	
	if (telnetEnabled && telnetPort > 0 && telnetPort < 65536) {
		[delegate frontendSettings:self validFrontend:frontend port:telnetPort];
	}
	else {
		[delegate frontendSettings:self invalidFrontend:frontend];
	}
}

// NSXMLParser Delegate

- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qualifiedName attributes:(NSDictionary *)attributeDict {
	key = [[attributeDict objectForKey:@"key"] retain];
	
	if ([key isEqualToString:@"NetworkControlEnabled"] || [key isEqualToString:@"NetworkControlPort"]) {
		currentString  = [[NSMutableString alloc] init];
	}
}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string {
	[currentString appendString:string];
}

- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qualifiedName {
	if ([key isEqualToString:@"NetworkControlEnabled"]) {
		telnetEnabled = [currentString boolValue];
		[currentString release];
		currentString = nil;
	}
	else if ([key isEqualToString:@"NetworkControlPort"]) {
		telnetPort = [currentString intValue];
		[currentString release];
		currentString = nil;
	}
	
	[key release];
	key = nil;
}
			 
- (void)dealloc {
	[connection cancel];
	[connection release];
	[parser abortParsing];
	[parser release];
	[key release];
	[currentString release];
	[settingsXML release];
	[delegate release];
	[frontend release];
	[super dealloc];
}

@end
