//
//  GuideResponse.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "Channel.h"
#import "GuideResponse.h"

@implementation GuideResponse

@synthesize programGuideData;
@synthesize retrievalDate;
@synthesize channels;
@synthesize startDate;
@synthesize endDate;
@synthesize guideMinuteLength;
@synthesize lastChanId;
@synthesize finished;

- (id)initWithProgramGuideData:(ProgramGuideData *)newProgramGuideData {
	if (self = [super init]) {
		programGuideData = newProgramGuideData;
		channels = [[NSMutableArray alloc] init];
		prevNumChannels = 0;
	}
	return self;
}

- (void)dealloc {
	[startDate release];
	[endDate release];
	[retrievalDate release];
	[channels release];
	[super dealloc];
}

// NSXMLParser Delegate

- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qualifiedName attributes:(NSDictionary *)attributesDict {
	if ([elementName isEqualToString:@"Channel"]) {
		if (channels.count < numChannels + prevNumChannels - 1 || numChannels == 1) {
			Channel *channel = [[[Channel alloc] initWithResponse:self attributes:attributesDict] autorelease];
			[channels addObject:channel];
			parser.delegate = channel;
		}
		else {
			prevNumChannels += numChannels - 1;
			lastChanId = [[attributesDict objectForKey:@"chanId"] intValue];
		}
	}
	else if ([elementName isEqualToString:@"StartTime"] || [elementName isEqualToString:@"EndTime"] || [elementName isEqualToString:@"NumOfChannels"]) {
		currentString  = [[NSMutableString alloc] init];
	}
}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string {
	[currentString appendString:string];
}

- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qualifiedName {
	if ([elementName isEqualToString:@"StartTime"]) {
		NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
		formatter.dateFormat = @"yyyy-MM-dd'T'HH:mm:ss";
		
		startDate = [[formatter dateFromString:currentString] retain];
		[formatter release];
		[currentString release];
		currentString = nil;
	}
	else if ([elementName isEqualToString:@"EndTime"]) {
		NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
		formatter.dateFormat = @"yyyy-MM-dd'T'HH:mm:ss";
		
		endDate = [[formatter dateFromString:currentString] retain];
		[formatter release];
		[currentString release];
		currentString = nil;
	}
	else if ([elementName isEqualToString:@"NumOfChannels"]) {
		numChannels = [currentString intValue];
		[currentString release];
		currentString = nil;
	}
	else if ([elementName isEqualToString:@"GetProgramGuideResponse"]) {
		guideMinuteLength = [[NSCalendar currentCalendar] components:(NSMinuteCalendarUnit) fromDate:startDate toDate:endDate options:0].minute;
		
		if (numChannels == 1) {
			for (Channel *channel in channels) {
				[channel getExtendedPrograms];
			}
			
			finished = YES;
		}
		
		parser.delegate = nil;
	}
}

- (void)parser:(NSXMLParser *)parser parseErrorOccurred:(NSError *)parseError {
	[currentString release];
	currentString = nil;
}

@end
