//
//  GuideScrollView.m
//  MyMote
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import <libkern/OSAtomic.h>

#import "defines.h"
#import "ChannelScrollView.h"
#import "GuideResponse.h"
#import "GuideScrollView.h"
#import "ProgramsView.h"
#import "ProgramGuideData.h"
#import "TimeScrollView.h"

@implementation GuideScrollView

@synthesize channelScrollView;
@synthesize timeScrollView;
@synthesize channelScrolling;
@synthesize timeScrolling;

- (id)initWithGuideData:(ProgramGuideData *)newData {
	if (self = [super init]) {
		self.autoresizingMask = UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleWidth;
		self.delegate = self;
		self.clearsContextBeforeDrawing = NO;
		self.opaque = YES;
		
		programsView = [[ProgramsView alloc] init];
		[self addSubview:programsView];
		
		data = [newData retain];
	}
	return self;
}

- (void)dataChanged {
	CGSize contentSize = CGSizeMake(EPG_PROGRAMS_WIDTH, data.response.channels.count * EPG_ROW_HEIGHT);
	if (self.contentSize.width != contentSize.width || self.contentSize.height != contentSize.height) {
		self.contentOffset = CGPointZero;
	}
	self.contentSize = contentSize;
	
	programsView.frame = CGRectMake(0, 0, contentSize.width, contentSize.height);
	programsView.data = data;
}

- (void)setChannelScrolling:(BOOL)newChannelScrolling {
	channelScrolling = newChannelScrolling;
	if (channelScrolling) {
		self.scrollEnabled = NO;
	}
	else if (!timeScrolling) {
		self.scrollEnabled = YES;
	}
}

- (void)setTimeScrolling:(BOOL)newTimeScrolling {
	timeScrolling = newTimeScrolling;
	if (timeScrolling) {
		self.scrollEnabled = NO;
	}
	else if (!channelScrolling) {
		self.scrollEnabled = YES;
	}
}

- (void)didReceiveMemoryWarning {
	[programsView releaseLayers];
}

- (void)dealloc {
	[data release];
	[channelScrollView release];
	[programsView release];
	[timeScrollView release];
	[super dealloc];
}

// UIScrollViewDelegate

- (void)scrollViewWillBeginDragging:(UIScrollView *)scrollView {
	channelScrollView.scrollEnabled = NO;
	timeScrollView.scrollEnabled = NO;
}

- (void)scrollViewDidScroll:(UIScrollView *)scrollView {
	channelScrollView.contentOffset = CGPointMake(channelScrollView.contentOffset.x, scrollView.contentOffset.y);
	timeScrollView.contentOffset = CGPointMake(scrollView.contentOffset.x, timeScrollView.contentOffset.y);
}

- (void)scrollViewDidEndDragging:(UIScrollView *)scrollView willDecelerate:(BOOL)decelerate {
	if (!decelerate) {
		channelScrollView.scrollEnabled = YES;
		timeScrollView.scrollEnabled = YES;
	}
}

- (void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView {
	channelScrollView.scrollEnabled = YES;
	timeScrollView.scrollEnabled = YES;
}

@end
