//
//  LIRCButtonEditor.m
//  MyMote
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "ButtonPickerController.h"
#import "LIRCButtonEditor.h"
#import "RemoteButton.h"

@implementation LIRCButtonEditor

- (id)initWithButton:(RemoteButton *)newButton pickerController:(ButtonPickerController *)newPickerController {
	if (self = [super init]) {
		self.title = @"LIRC Button";
		pickerController = [newPickerController retain];
		if (newButton) {
			button = [newButton retain];
			new = NO;
		}
		else {
			button = [[RemoteButton alloc] initWithInfo:[NSDictionary dictionary]];
			new = YES;
		}
	}
	return self;
}

- (void)loadView {
	UIBarButtonItem *cancelItem = [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemCancel target:self action:@selector(dismiss)] autorelease];
	self.navigationItem.leftBarButtonItem = cancelItem;
	if (!new) {
		UIBarButtonItem *doneItem = [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(done)] autorelease];
		self.navigationItem.rightBarButtonItem = doneItem;
	}
	
	self.view = [[[UIView alloc] init] autorelease];
	self.view.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
	self.view.backgroundColor = [UIColor groupTableViewBackgroundColor];
	
	UILabel *label;
	
	label = [[[UILabel alloc] initWithFrame:CGRectMake(5, 5, self.parentViewController.view.bounds.size.width - 10, 20)] autorelease];
	label.text = @"Name:";
	label.backgroundColor = [UIColor clearColor];
	[self.view addSubview:label];

	name = [[[UITextField alloc] initWithFrame:CGRectMake((self.parentViewController.view.bounds.size.width - 250) / 2, 30, 250, 30)] autorelease];
	name.text = [button.info objectForKey:@"Title"];
	name.borderStyle = UITextBorderStyleRoundedRect;
	name.delegate = self;
	[self.view addSubview:name];
	
	label = [[[UILabel alloc] initWithFrame:CGRectMake(5, 65, self.parentViewController.view.bounds.size.width - 10, 20)] autorelease];
	label.text = @"LIRC Remote Button Name:";
	label.backgroundColor = [UIColor clearColor];
	[self.view addSubview:label];
	
	lircButton = [[[UITextField alloc] initWithFrame:CGRectMake((self.parentViewController.view.bounds.size.width - 250) / 2, 90, 250, 30)] autorelease];
	lircButton.text = [button.info objectForKey:@"LIRC Button"];
	lircButton.borderStyle = UITextBorderStyleRoundedRect;
	lircButton.autocapitalizationType = UITextAutocapitalizationTypeNone;
	lircButton.autocorrectionType = UITextAutocorrectionTypeNo;
	lircButton.delegate = self;
	[self.view addSubview:lircButton];
	
	if (!new) {
		delete = [UIButton buttonWithType:UIButtonTypeCustom];
		delete.frame = CGRectMake((self.parentViewController.view.bounds.size.width - 310) / 2, 315, 310, 45);
		[delete setBackgroundImage:[UIImage imageNamed:@"delete.png"] forState:UIControlStateNormal];
		[delete setBackgroundImage:[UIImage imageNamed:@"deleteHighlighted.png"] forState:UIControlStateHighlighted];
		[delete setTitle:@"Delete LIRC Button" forState:UIControlStateNormal];
		[delete addTarget:self action:@selector(delete) forControlEvents:UIControlEventTouchUpInside];
		[self.view addSubview:delete];
	}
}

- (void)dismiss {
	[self.parentViewController dismissModalViewControllerAnimated:YES];
}

- (void)done {
	NSMutableDictionary *LIRCButtons = [NSMutableDictionary dictionaryWithDictionary:[[NSUserDefaults standardUserDefaults] objectForKey:@"LIRC Buttons"]];
	[LIRCButtons setObject:[NSMutableArray arrayWithArray:[LIRCButtons objectForKey:@"Buttons"]] forKey:@"Buttons"];
	
	int index = -1;
	for (NSDictionary *info in [LIRCButtons objectForKey:@"Buttons"]) {
		if (new || ![name.text isEqualToString:[button.info objectForKey:@"Title"]]) {
			if ([name.text isEqualToString:[info objectForKey:@"Title"]]) {
				[[[[UIAlertView alloc] initWithTitle:@"Name Exists" message:@"A button with the given name already exists." delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil] autorelease] show];
				return;
			}
		}
		
		if (!new) {
			if ([[button.info objectForKey:@"Title"] isEqualToString:[info objectForKey:@"Title"]]) {
				index = [[LIRCButtons objectForKey:@"Buttons"] indexOfObject:info];
			}
		}
	}

	[button.info setObject:@"LIRC" forKey:@"Context"];
	[button.info setObject:name.text forKey:@"Title"];
	[button.info setObject:lircButton.text forKey:@"LIRC Button"];
	
	if (!new) {
		[[LIRCButtons objectForKey:@"Buttons"] replaceObjectAtIndex:index withObject:button.info];
		[[NSUserDefaults standardUserDefaults] setObject:LIRCButtons forKey:@"LIRC Buttons"];
		[button setNeedsDisplay];
	}
	else {
		[pickerController addLIRCButton:button];
	}
	
	[self dismiss];
}

- (void)delete {
	[pickerController deleteLIRCButton:button];
	[self dismiss];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	return NO;
}

- (void)dealloc {
	[button release];
	[pickerController release];
	[super dealloc];
}

// UITextFieldDelegate

- (void)textFieldDidBeginEditing:(UITextField *)textField {
	[self.navigationItem setRightBarButtonItem:nil animated:YES];
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
	[textField resignFirstResponder];
	
	if ([name.text length] && [lircButton.text length] && !self.navigationItem.rightBarButtonItem) {
		UIBarButtonItem *doneItem = [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(done)] autorelease];
		[self.navigationItem setRightBarButtonItem:doneItem animated:YES];
	}
	else if (![name.text length] || ![lircButton.text length]) {
		[self.navigationItem setRightBarButtonItem:nil animated:YES];
	}
	
	return YES;
}

@end
