//
//  MyMoteAppDelegate.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import <CoreTelephony/CTCall.h>
#import <CoreTelephony/CTCallCenter.h>

#import "defines.h"
#import "BackendSelect.h"
#import "Keyboard.h"
#import "MainTabBarController.h"
#import "MythXMLParser.h";
#import "MyMoteAppDelegate.h"
#import "ProgramGuide.h"
#import "Recordings.h"
#import "Remote.h"
#import "Settings.h"
#import "TelnetRemote.h"

@implementation MyMoteAppDelegate

@synthesize operationQueue;
@synthesize debugInfo;
@synthesize tabBarController;
@synthesize remote;
@synthesize programGuide;
@synthesize telnetRemote;
@synthesize settings;
@synthesize backendLocator;
@synthesize hiddenTextField;

- (void)applicationDidFinishLaunching:(UIApplication *)application {
	telnetRemote = [[TelnetRemote alloc] init];
	[DEFAULTS registerDefaults:[NSDictionary dictionaryWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"Defaults" ofType:@"plist"]]];
	
	[UIApplication sharedApplication].idleTimerDisabled = [[NSUserDefaults standardUserDefaults] boolForKey:@"Disable Auto-Lock"];
	
	debugInfo = [[NSMutableString alloc] init];
	
	operationQueue = [[NSOperationQueue alloc] init];
	
	backendLocator = [[BackendLocator alloc] init];

	window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
	[window makeKeyAndVisible];
    
    tabBarController = [[MainTabBarController alloc] init];
	
	[MythXMLParser initMythXMLParser];
	
	remote = [[[Remote alloc] init] autorelease];
	programGuide = [[[ProgramGuide alloc] init] autorelease];
	recordings = [[[Recordings alloc] init] autorelease];
	settings = [[[Settings alloc] init] autorelease];
	
	tabBarController.viewControllers = [NSArray arrayWithObjects:remote, programGuide, recordings, settings, nil];
	
	[window addSubview:tabBarController.view];
	
	keyboard = [[Keyboard alloc] init];
	hiddenTextField = [[[UITextField alloc] initWithFrame:CGRectMake(0, 0, 100, 20)] autorelease];
	hiddenTextField.hidden = YES;
	hiddenTextField.text = @" ";
	hiddenTextField.delegate = keyboard;
	[window addSubview:hiddenTextField];
	
	if (NSClassFromString(@"CTCallCenter")) {
		callCenter = [[CTCallCenter alloc] init];
		callCenter.callEventHandler = ^(CTCall *call) {
			if (call.callState == CTCallStateIncoming && [DEFAULTS boolForKey:@"Pause When Called"]) {
				[TELNETREMOTE sendCommand:@"key PAUSE"];
			}
		};
	}
	
	if (!BACKEND || !FRONTEND || !FRONTEND_PORT) {
		BackendSelect *backendSelect = [[[BackendSelect alloc] init] autorelease];
		UINavigationController *navigationController = [[[UINavigationController alloc] initWithRootViewController:backendSelect] autorelease];
		navigationController.navigationBar.barStyle = UIBarStyleBlackOpaque;
		[tabBarController presentModalViewController:navigationController animated:YES];
	}
}

- (void)applicationDidBecomeActive:(UIApplication *)application {
	if (BACKEND && FRONTEND && FRONTEND_PORT) {
		TELNETREMOTE.resuming = YES;
		[TELNETREMOTE setup];
	}
}

- (void)dealloc {
	[callCenter release];
	[keyboard release];
	[operationQueue release];
	[tabBarController release];
	[window release];
	[telnetRemote release];
	[debugInfo release];
	[super dealloc];
}

@end