//
//  NSString+DrawingSupport.m
//  MyMote
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "NSString+DrawingSupport.h"


@implementation NSString (DrawingSupport)

- (void)drawVerticallyCenteredInRect:(CGRect)rect withFont:(UIFont *)font {
	CGSize stringSize = [self sizeWithFont:font constrainedToSize:rect.size];
	CGRect stringRect = CGRectMake(rect.origin.x, rect.origin.y + (rect.size.height - stringSize.height) / 2, stringSize.width, stringSize.height);
	[self drawInRect:stringRect withFont:font lineBreakMode:UILineBreakModeTailTruncation];
}

- (void)drawCenteredInRect:(CGRect)rect withFont:(UIFont *)font {
	CGSize stringSize = [self sizeWithFont:font constrainedToSize:rect.size];
	CGRect stringRect = CGRectMake(rect.origin.x + (rect.size.width - stringSize.width) / 2, rect.origin.y + (rect.size.height - stringSize.height) / 2, stringSize.width, stringSize.height);
	[self drawInRect:stringRect withFont:font lineBreakMode:UILineBreakModeTailTruncation];
}

- (void)drawCenteredFitInRect:(CGRect)rect withFont:(UIFont *)font {
	CGFloat fontSize;
	CGSize stringSize = [self sizeWithFont:font minFontSize:0 actualFontSize:&fontSize forWidth:rect.size.width lineBreakMode:UILineBreakModeClip];
	CGRect stringRect = CGRectMake(rect.origin.x + (rect.size.width - stringSize.width) / 2, rect.origin.y + (rect.size.height - stringSize.height) / 2, stringSize.width, stringSize.height);
	[self drawInRect:stringRect withFont:[font fontWithSize:fontSize]];	
}

- (void)drawCenteredInRect:(CGRect)rect withFont:(UIFont *)font context:(CGContextRef)ctx {
	CGSize stringSize = [self sizeWithFont:font constrainedToSize:rect.size];
	CGRect stringRect = CGRectMake(rect.origin.x + (rect.size.width - stringSize.width) / 2, rect.origin.y + (rect.size.height - stringSize.height) / 2, stringSize.width, stringSize.height);
	
	CGContextSelectFont(ctx, [font.fontName UTF8String], font.pointSize, kCGEncodingMacRoman);
	CGContextShowTextAtPoint(ctx, stringRect.origin.x, stringRect.origin.y + stringRect.size.height / 4 /* I have no clue why this is needed */, [self UTF8String], [self length]);
}

- (void)drawCenteredFitInRect:(CGRect)rect withFont:(UIFont *)font context:(CGContextRef)ctx {
	CGFloat fontSize;
	CGSize stringSize = [self sizeWithFont:font minFontSize:0 actualFontSize:&fontSize forWidth:rect.size.width lineBreakMode:UILineBreakModeClip];
	CGRect stringRect = CGRectMake(rect.origin.x + (rect.size.width - stringSize.width) / 2, rect.origin.y + (rect.size.height - stringSize.height) / 2, stringSize.width, stringSize.height);
	
	CGContextSelectFont(ctx, [font.fontName UTF8String], fontSize, kCGEncodingMacRoman);
	CGContextShowTextAtPoint(ctx, stringRect.origin.x, stringRect.origin.y + stringRect.size.height / 4 /* I have no clue why this is needed */, [self UTF8String], [self length]);
}

@end
