//
//  NSUserDefaults+BackendSupport.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "NSUserDefaults+BackendSupport.h"

@implementation NSUserDefaults (BackendSupport)

- (void)setBackend:(Backend *)backend forKey:(NSString *)key {
	[self setObject:[NSKeyedArchiver archivedDataWithRootObject:backend] forKey:key];
}

- (Backend *)backendForKey:(NSString *)key {
	NSData *data = [self dataForKey:key];
	if (data) {
		return [NSKeyedUnarchiver unarchiveObjectWithData:data];
	}
	return nil;
}

- (void)setBackendArray:(NSArray *)array forKey:(NSString *)key {
	NSMutableArray *mutableArray = [NSMutableArray arrayWithCapacity:[array count]];
	for (Backend *backend in array) {
		[mutableArray addObject:[NSKeyedArchiver archivedDataWithRootObject:backend]];
	}
	[self setObject:mutableArray forKey:key];
}

- (NSArray *)backendArrayForKey:(NSString *)key {
	NSMutableArray *array = [NSMutableArray array];
	for (NSData *data in [self arrayForKey:key]) {
		[array addObject:[NSKeyedUnarchiver unarchiveObjectWithData:data]];
	}
	return array;
}

@end
