//
//  Program.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "defines.h"
#import "Channel.h"
#import "GuideResponse.h"
#import "Program.h"
#import "ProgramGuideData.h"
#import "Regexp.h"

@implementation Program

@synthesize channel;
@synthesize startDate;
@synthesize endDate;
@synthesize programMinuteLength;
@synthesize description;
@synthesize color;

- (id)initWithChannel:(id)newChannel attributes:(NSDictionary *)newAttributes {
	if (self = [super init]) {
		static NSDateFormatter *formatter = nil;
		if (!formatter) {
			formatter = [[NSDateFormatter alloc] init];
			formatter.dateFormat = @"yyyy-MM-dd'T'HH:mm:ss";
		}
		
		channel = newChannel;
		attributes = [newAttributes retain];

		startDate = [[formatter dateFromString:self.startTime] retain];
		endDate = [[formatter dateFromString:self.endTime] retain];
		
		programMinuteLength = [[NSCalendar currentCalendar] components:(NSMinuteCalendarUnit) fromDate:startDate toDate:endDate options:0].minute;
		
		currentString = [[NSMutableString alloc] init];
	}
	return self;
}

ATTRIBUTE_PROPERTY(title, @"title");
ATTRIBUTE_PROPERTY(subtitle, @"subTitle");
ATTRIBUTE_PROPERTY(type, @"catType");
ATTRIBUTE_PROPERTY(category, @"category");
ATTRIBUTE_PROPERTY(startTime, @"startTime");
ATTRIBUTE_PROPERTY(endTime, @"endTime");

- (Boolean)repeat {
	return [[attributes objectForKey:(@"repeat")] boolValue];
}

- (void)drawInRect:(CGRect)rect {
	CGContextRef ctx = UIGraphicsGetCurrentContext();
	CGContextAddRect(ctx, rect);
	
	if (color) {
		CGContextSetFillColorWithColor(ctx, color.CGColor);
	}
	else {
		CGContextSetRGBFillColor(ctx, (float)48 / 255, (float)48 / 255, (float)48 / 255, 1);
	}
	CGContextFillPath(ctx);
	
	CGContextSetRGBFillColor(ctx, 1, 1, 1, 1);
	[self.title drawVerticallyCenteredInRect:CGRectMake(rect.origin.x + 1, rect.origin.y + 1, rect.size.width - 2, rect.size.height - 2) withFont:[UIFont systemFontOfSize:([UIFont systemFontSize] * SCALE)]];
}

- (void)dealloc {
	[startDate release];
	[endDate release];
	[attributes release];
	[currentString release];
	[description release];
	[super dealloc];
}

- (NSComparisonResult)programSort:(Program *)other {
	return [self.startDate compare:other.startDate];
}

// NSXMLParser Delegate

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string {
	[currentString appendString:string];
}

- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qualifiedName {
	if (currentString) {
		description = [[currentString stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]] retain];
		[currentString release];
		currentString = nil;
	}
	
	NSEnumerator *enumerator = [channel.response.programGuideData.categories objectEnumerator];
	NSDictionary *category;
	while (category = [enumerator nextObject]) {
		if ([[category objectForKey:@"Regexp"] test:self.category]) {
			color = [category objectForKey:@"UIColor"];
			break;
		}
	}
	
	parser.delegate = channel;
}

- (void)parser:(NSXMLParser *)parser parseErrorOccurred:(NSError *)parseError {
	[currentString release];
	currentString = nil;
	parser.delegate = channel;
}

@end
