//
//  ProgramGuide.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "defines.h"
#import "ProgramGuide.h"
#import "ProgramGuideView.h"
#import "MainTabBarController.h"

@implementation ProgramGuide

@synthesize programGuideView;

- (id)init
{
	if (self = [super init]) {
		self.title = @"Program Guide";
		self.tabBarItem.image = [UIImage imageNamed:@"guide.png"];

		if (BACKEND) {
			[self loadView];
		}
	}
	return self;
}

- (void)loadView {
	programGuideView = [[[ProgramGuideView alloc] initWithFrame:[UIScreen mainScreen].applicationFrame] autorelease];
	self.view = programGuideView;
	[programGuideView updateProgramGuideWithDate:nil];
}

id findView(NSString *className, UIView *superview) {
	for (UIView *subview in superview.subviews) {
		if ([subview isMemberOfClass:NSClassFromString(className)]) {
			return subview;
		}
		else {
			id possibility = findView(className, subview);
			if (possibility) {
				return possibility;
			}
		}
	}
	return nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
	static UIInterfaceOrientation currentOrientation = UIInterfaceOrientationPortrait;
	
	UITabBar *tabBar = (UITabBar *)TAB_BAR_CONTROLLER.rotatingFooterView;
	UIView *transitionView = findView(@"UITransitionView", [UIApplication sharedApplication].keyWindow);
	if (interfaceOrientation == UIInterfaceOrientationPortrait && currentOrientation != UIInterfaceOrientationPortrait) {
		CGRect frame = transitionView.frame;
		frame.size.height -= tabBar.bounds.size.height;
		transitionView.frame = frame;
	}
	else if (interfaceOrientation != UIInterfaceOrientationPortrait && currentOrientation == UIInterfaceOrientationPortrait) {
		CGRect frame = transitionView.frame;
		frame.size.height += tabBar.bounds.size.height;
		transitionView.frame = frame;
	}
	
	currentOrientation = interfaceOrientation;

	return YES;
}

- (void)didReceiveMemoryWarning {
	[super didReceiveMemoryWarning];
	[programGuideView didReceiveMemoryWarning];
}

- (void)willAnimateFirstHalfOfRotationToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration {
	[programGuideView willRotate];
}

- (void)willAnimateSecondHalfOfRotationFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation duration:(NSTimeInterval)duration {
	[programGuideView didRotate];
	TAB_BAR_CONTROLLER.rotatingFooterView.hidden = (self.interfaceOrientation != UIInterfaceOrientationPortrait);
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation {
	TAB_BAR_CONTROLLER.rotatingFooterView.hidden = (self.interfaceOrientation != UIInterfaceOrientationPortrait);
}

@end