//
//  ProgramGuideData.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "defines.h"
#import "Backend.h"
#import "Channel.h"
#import "GuideResponse.h"
#import "MythXMLParser.h"
#import "Program.h"
#import "ProgramGuideData.h"
#import "ProgramGuideView.h"
#import "Regexp.h"

NSInteger channelSort(id a, id b, void *unused) {
	if ([[a number] intValue] < [[b number] intValue]) {
		return NSOrderedAscending;
	}
	if ([[a number] intValue] > [[b number] intValue]) {
		return NSOrderedDescending;
	}
	return NSOrderedSame;
}

@implementation ProgramGuideData

@synthesize categories;
@synthesize response;
@synthesize channelIcons;

- (id)initWithProgramGuideView:(ProgramGuideView *)newProgramGuideView {
	if (self = [super init]) {
		programGuideView = [newProgramGuideView retain];
		
		NSString *errString;
		categories = [[NSPropertyListSerialization propertyListFromData:[NSData dataWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"ProgramCategories" ofType:@"plist"]] mutabilityOption:NSPropertyListMutableContainersAndLeaves format:NULL errorDescription:&errString] retain];
		if (errString) {
			NSLog(@"ProgramCategories.plist failed parsing: %@", errString);
			[errString release];
		}
		
		for (NSString *key in categories) {
			NSMutableDictionary *category = [categories objectForKey:key];
			Regexp *regexp = [[[Regexp alloc] initWithExpression:[category objectForKey:@"Regular Expression"]] autorelease];
			if (regexp) {
				[category setObject:regexp forKey:@"Regexp"];
				
				unsigned char red;
				unsigned char blue;
				unsigned char green;
				if (sscanf([[category objectForKey:@"Color"] UTF8String], "%2hhx%2hhx%2hhx", &red, &green, &blue) == 3) {
					[category setObject:[UIColor colorWithRed:((float)red / 255) green:((float)green / 255) blue:((float)blue / 255) alpha:1] forKey:@"UIColor"];
				}
				else {
					[categories removeObjectForKey:key];
				}
			}
			else {
				[categories removeObjectForKey:key];
			}
		}
		
		channelIcons = [[NSMutableDictionary alloc] init];
	}
	return self;
}

- (void)getDataWithDate:(NSDate *)date {
	if (!date) {
		date = [NSDate date];
	}
	
	NSDateComponents *components = [[NSCalendar currentCalendar] components:(NSYearCalendarUnit | NSMonthCalendarUnit | NSDayCalendarUnit | NSHourCalendarUnit | NSMinuteCalendarUnit) fromDate:date];
	
	components.minute = components.minute < 30 ? 0 : 30;
	
	[startDate release];
	startDate = [[[NSCalendar currentCalendar] dateFromComponents:components] retain];
	
	NSDateComponents *sixHours = [[[NSDateComponents alloc] init] autorelease];
	sixHours.hour = 6;
	
	[endDate release];
	endDate = [[[NSCalendar currentCalendar] dateByAddingComponents:sixHours toDate:startDate options:0] retain];
	
	NSDateFormatter *formatter = [[[NSDateFormatter alloc] init] autorelease];
	formatter.dateFormat = @"yyyy-MM-dd'T'HH:mm:ss";
	
	NSString *url = [[NSString alloc] initWithFormat:@"http://%@/Myth/GetProgramGuide?StartTime=%@&EndTime=%@&NumOfChannels=-1&Details=1", BACKEND_LOCATION, [formatter stringFromDate:startDate], [formatter stringFromDate:endDate]];
	MythXMLParser *parser = [[MythXMLParser alloc] initWithContentsOfURL:[NSURL URLWithString:url]];
	parser.delegate = self;
	
	[response release];
	response = [[GuideResponse alloc] initWithProgramGuideData:self];
	
	do {
		if ([parser parse] == NO) {
			[response release];
			response = nil;
			NSLog(@"Error Retrieving Guide Data: %@", [[parser parserError] localizedDescription]);
			NSLog(@"URL Used: %@", url);
			[parser release];
			[url release];
			[[[[UIAlertView alloc] initWithTitle:@"Could Not Retrieve Program Guide" message:@"Please verify that the MythTV backend is running." delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil] autorelease] show];
			return;
		}
		
		[url release];
		url = [[NSString alloc] initWithFormat:@"http://%@/Myth/GetProgramGuide?StartTime=%@&EndTime=%@&NumOfChannels=-1&Details=1&StartChanId=%u", BACKEND_LOCATION, [formatter stringFromDate:startDate], [formatter stringFromDate:endDate], response.lastChanId];
		[parser release];
		parser = [[MythXMLParser alloc] initWithContentsOfURL:[NSURL URLWithString:url]];
		parser.delegate = self;
	} while (!response.finished);

	[parser release];
	[url release];
	[response.channels sortUsingFunction:&channelSort context:nil];
	[programGuideView drawContentView];
}

- (void)dealloc {
	[programGuideView release];
	[categories release];
	[response release];
	[channelIcons release];
	[super dealloc];
}

// NSXMLParser Delegate

- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qualifiedName attributes:(NSDictionary *)attributeDict {
	if ([elementName isEqualToString:@"GetProgramGuideResponse"]) {
		parser.delegate = response;
	}
}

@end
