//
//  ProgramGuideView.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "defines.h"
#import "ChangeTimeView.h"
#import "ChannelScrollView.h"
#import "DateView.h"
#import "GuideResponse.h"
#import "GuideScrollView.h"
#import "ProgramGuideData.h"
#import "ProgramGuideView.h"
#import "ProgramInfoView.h"
#import "TimeScrollView.h"

@implementation ProgramGuideView

@synthesize programGuideData;

- (id)initWithFrame:(CGRect)frame {
    if (self = [super initWithFrame:frame]) {
		self.autoresizesSubviews = YES;
		self.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
		self.clearsContextBeforeDrawing = NO;
		self.opaque = YES;
		
		activityIndicator = [[[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhiteLarge] autorelease];
		activityIndicator.frame = CGRectMake(0, 0, 50, 50);
		activityIndicator.center = self.center;
		activityIndicator.autoresizingMask = UIViewAutoresizingFlexibleTopMargin | UIViewAutoresizingFlexibleBottomMargin | UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin;
		activityIndicator.hidesWhenStopped = YES;
		[self addSubview:activityIndicator];

		programGuideData = [[ProgramGuideData alloc] initWithProgramGuideView:self];
		dateView = [[DateView alloc] initWithFrame:CGRectMake(1, 1, EPG_CHANNEL_WIDTH - 2, EPG_ROW_HEIGHT - 2) data:programGuideData];
		guideScrollView = [[GuideScrollView alloc] initWithGuideData:programGuideData];
		channelScrollView = [[ChannelScrollView alloc] initWithGuideData:programGuideData];
		timeScrollView = [[TimeScrollView alloc] initWithGuideData:programGuideData];
		guideScrollView.channelScrollView = channelScrollView;
		guideScrollView.timeScrollView = timeScrollView;
		channelScrollView.guideScrollView = guideScrollView;
		timeScrollView.guideScrollView = guideScrollView;
		changeTimeView = [[ChangeTimeView alloc] initWithProgramGuideView:self];
		programInfoView = [[ProgramInfoView alloc] initWithProgramGuideView:self];
	}
    return self;
}

- (void)drawContentView {
	[dateView setNeedsDisplay];
	[guideScrollView dataChanged];
	[channelScrollView dataChanged];
	[timeScrollView dataChanged];
	guideScrollView.frame = CGRectMake(EPG_CHANNEL_WIDTH, EPG_ROW_HEIGHT, self.bounds.size.width - EPG_CHANNEL_WIDTH, self.bounds.size.height - EPG_ROW_HEIGHT);
	channelScrollView.frame = CGRectMake(0, EPG_ROW_HEIGHT, EPG_CHANNEL_WIDTH, self.bounds.size.height - EPG_ROW_HEIGHT);
	timeScrollView.frame = CGRectMake(EPG_CHANNEL_WIDTH, 0, self.bounds.size.width - EPG_CHANNEL_WIDTH, EPG_ROW_HEIGHT);
	[self addSubview:guideScrollView];
	[self addSubview:channelScrollView];
	[self addSubview:timeScrollView];
	[self addSubview:dateView];
	[activityIndicator stopAnimating];
}

- (void)updateProgramGuideWithDate:(NSDate *)date {
	[startDate release];
	startDate = [date retain];
	[activityIndicator startAnimating];
	NSInvocationOperation *operation = [[NSInvocationOperation alloc] initWithTarget:programGuideData selector:@selector(getDataWithDate:) object:date];
	[OPERATION_QUEUE addOperation:operation];
	[operation release];
}

- (void)requestNewTime {
	if (programInfoView.superview) {
		[self closeProgramInfo];
	}

	[changeTimeView showWithFrame:self.bounds];
	[self addSubview:changeTimeView];
	CGPoint center = changeTimeView.center;
	center.y -= changeTimeView.bounds.size.height;
	[UIView beginAnimations:nil context:NULL];
	changeTimeView.center = center;
	[UIView commitAnimations];
}

- (void)changeTime {
	[guideScrollView removeFromSuperview];
	[channelScrollView removeFromSuperview];
	[timeScrollView removeFromSuperview];
	[dateView removeFromSuperview];
	NSNumber *timeChanged = [[NSNumber numberWithBool:true] retain];
	CGPoint center = changeTimeView.center;
	center.y += EPG_NAV_BAR_HEIGHT + changeTimeView.datePicker.bounds.size.height;
	[UIView beginAnimations:nil context:timeChanged];
	[UIView setAnimationDelegate:self];
	[UIView setAnimationDidStopSelector:@selector(newTimeViewGone:finished:context:)];
	changeTimeView.center = center;
	[UIView commitAnimations];
}

- (void)cancelNewTimeView {
	NSNumber *timeChanged = [[NSNumber numberWithBool:false] retain];
	CGPoint center = changeTimeView.center;
	center.y += EPG_NAV_BAR_HEIGHT + changeTimeView.datePicker.bounds.size.height;
	[UIView beginAnimations:nil context:timeChanged];
	[UIView setAnimationDelegate:self];
	[UIView setAnimationDidStopSelector:@selector(newTimeViewGone:finished:context:)];
	changeTimeView.center = center;
	[UIView commitAnimations];
}

- (void)newTimeViewGone:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context {
	[changeTimeView removeFromSuperview];
	if ([(NSNumber *)context boolValue]) {
		[self updateProgramGuideWithDate:changeTimeView.datePicker.date];
	}
	[(NSNumber *)context release];
}

- (void)showProgramInfo:(Program *)program {
	BOOL shown = (programInfoView.superview != nil);
	
	if (changeTimeView.superview) {
		[self cancelNewTimeView];
	}
	
	if (program) {
		programInfoProgram = program;
	}
	else {
		program = programInfoProgram;
	}
	
	programInfoView.program = program;
	
	if (UIInterfaceOrientationIsPortrait([UIApplication sharedApplication].statusBarOrientation)) {
		programInfoView.frame = CGRectMake(self.bounds.origin.x, self.bounds.origin.y + self.bounds.size.height - (shown ? 210 : 0), self.bounds.size.width, 210);
	}
	else {
		programInfoView.frame = CGRectMake(self.bounds.origin.x + self.bounds.size.width - (shown ? self.bounds.size.height : 0), self.bounds.origin.y, self.bounds.size.height, self.bounds.size.height);
	}

	if (!programInfoView.superview) {
		[self addSubview:programInfoView];
		CGPoint center = programInfoView.center;
		if (UIInterfaceOrientationIsPortrait([UIApplication sharedApplication].statusBarOrientation)) {
			center.y -= programInfoView.bounds.size.height;
		}
		else {
			center.x -= programInfoView.bounds.size.width;
		}
		[UIView beginAnimations:nil context:NULL];
		programInfoView.center = center;
		[UIView commitAnimations];
	}
}

- (void)closeProgramInfo {
	CGPoint center = programInfoView.center;
	if (UIInterfaceOrientationIsPortrait([UIApplication sharedApplication].statusBarOrientation)) {
		center.y += programInfoView.bounds.size.height;
	}
	else {
		center.x += programInfoView.bounds.size.width;
	}
	[UIView beginAnimations:nil context:nil];
	[UIView setAnimationDelegate:self];
	[UIView setAnimationDidStopSelector:@selector(programInfoViewGone:finished:context:)];
	programInfoView.center = center;
	[UIView commitAnimations];
}

- (void)programInfoViewGone:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context {
	[programInfoView removeFromSuperview];
}

- (void)willRotate {
	if (changeTimeView.superview) {
		[self cancelNewTimeView];
		afterRotateCall = @selector(requestNewTime);
	}
	else if (programInfoView.superview) {
		[self closeProgramInfo];
		afterRotateCall = @selector(showProgramInfo:);
	}
	else {
		afterRotateCall = nil;
	}
}

- (void)didRotate {
	if (afterRotateCall) {
		[self performSelector:afterRotateCall withObject:nil];
	}
}

- (void)didReceiveMemoryWarning {
	[guideScrollView didReceiveMemoryWarning];
}

- (void)dealloc
{
	[activityIndicator release];
	[programInfoView release];
	[changeTimeView release];
	[programGuideData release];
	[guideScrollView release];
	[channelScrollView release];
	[timeScrollView release];
	[dateView release];
	[startDate release];
	[super dealloc];
}

@end
