//
//  ProgramInfoView.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "defines.h"
#import "BackgroundView.h"
#import "Channel.h"
#import "GuideResponse.h"
#import "Program.h"
#import "ProgramGuideData.h"
#import "ProgramInfoView.h"
#import "TelnetRemote.h"

@implementation ProgramInfoView

@synthesize program;

- (id)initWithProgramGuideView:(ProgramGuideView *)programGuideView {
    if (self = [super initWithFrame:CGRectMake(0, 0, 320, 210)]) {
        self.backgroundColor = [UIColor groupTableViewBackgroundColor];
		self.autoresizesSubviews = YES;
		
		UINavigationBar *navBar = [[[UINavigationBar alloc] init] autorelease];
		navBar.frame = CGRectMake(0, 0, self.bounds.size.width, EPG_NAV_BAR_HEIGHT);
		navBar.autoresizingMask = UIViewAutoresizingFlexibleWidth;
		[self addSubview:navBar];
		
		UINavigationItem *navItem = [[[UINavigationItem alloc] init] autorelease];
		navItem.leftBarButtonItem = [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemCancel target:programGuideView action:@selector(closeProgramInfo)] autorelease];
		navItem.rightBarButtonItem = [[[UIBarButtonItem alloc] initWithTitle:@"Go To Channel" style:UIBarButtonItemStylePlain target:self action:@selector(goToChannel:)] autorelease];
		
		[navBar pushNavigationItem:navItem animated:NO];
		
		BackgroundView *backgroundView = [[[BackgroundView alloc] initWithFrame:CGRectMake(self.bounds.size.width - 5 - EPG_CHANNEL_ICON_DIM - 10, EPG_NAV_BAR_HEIGHT + 5, EPG_CHANNEL_ICON_DIM + 10, EPG_CHANNEL_ICON_DIM + 35)] autorelease];
		backgroundView.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleBottomMargin;
		[self addSubview:backgroundView];
		
		channelCallSign = [[UILabel alloc] initWithFrame:CGRectMake(5, 5, backgroundView.bounds.size.width - 10, EPG_CHANNEL_ICON_DIM)];
		channelCallSign.textAlignment = UITextAlignmentCenter;
		channelCallSign.adjustsFontSizeToFitWidth = YES;
		channelCallSign.backgroundColor = [UIColor clearColor];
		[backgroundView addSubview:channelCallSign];
		
		channelIcon = [[UIImageView alloc] initWithFrame:CGRectMake(5, 5, backgroundView.bounds.size.width - 10, EPG_CHANNEL_ICON_DIM)];
		channelIcon.backgroundColor = [UIColor clearColor];
		[backgroundView addSubview:channelIcon];
		
		channelNumber = [[UILabel alloc] initWithFrame:CGRectMake(5, 10 + EPG_CHANNEL_ICON_DIM, backgroundView.bounds.size.width - 10, 20)];
		channelNumber.textAlignment = UITextAlignmentCenter;
		channelNumber.adjustsFontSizeToFitWidth = YES;
		channelNumber.backgroundColor = [UIColor clearColor];
		[backgroundView addSubview:channelNumber];
		
		titleText = [[UILabel alloc] initWithFrame:CGRectMake(5, EPG_NAV_BAR_HEIGHT + 5, self.bounds.size.width - EPG_CHANNEL_ICON_DIM - 10, 50)];
		titleText.font = [UIFont systemFontOfSize:20];
		titleText.numberOfLines = 0;
		titleText.backgroundColor = [UIColor clearColor];
		titleText.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleBottomMargin;
		[self addSubview:titleText];

		metaText = [[UILabel alloc] initWithFrame:CGRectMake(5, EPG_NAV_BAR_HEIGHT + 10 + 50, self.bounds.size.width - 10, 50)];
		metaText.numberOfLines = 0;
		metaText.backgroundColor = [UIColor clearColor];
		metaText.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleTopMargin | UIViewAutoresizingFlexibleBottomMargin;
		[self addSubview:metaText];
		
		subtitleText = [[UITextView alloc] initWithFrame:CGRectMake(5, EPG_NAV_BAR_HEIGHT + 50 + 50 + 10, self.bounds.size.width - 10, self.bounds.size.height - EPG_NAV_BAR_HEIGHT - 50 - 50 - 15)];
		subtitleText.font = [UIFont systemFontOfSize:17];
		subtitleText.editable = NO;
		subtitleText.backgroundColor = [UIColor clearColor];
		subtitleText.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleTopMargin;
		[self addSubview:subtitleText];
    }
    return self;
}

- (void)setProgram:(Program *)newProgram {
	[program release];
	program = [newProgram retain];
	
	Channel *channel = program.channel;
	
	NSDateFormatter *formatter = [[[NSDateFormatter alloc] init] autorelease];
	formatter.dateFormat = @"h:mm a";
	
	channum = channel.number;
	titleText.text = program.title;
	metaText.text = [NSString stringWithFormat:@"%@ - %@%@\nCategory: %@", [formatter stringFromDate:program.startDate], [formatter stringFromDate:program.endDate], (program.repeat ? @" (R)" : @""), (program.category.length ? program.category : @"None")];
	channelCallSign.text = channel.callSign;
	
	UIImage *icon = [program.channel.response.programGuideData.channelIcons objectForKey:program.channel.callSign];
	if (icon && icon != (UIImage *)[NSNull null]) {
		channelIcon.image = icon;
	}
	else {
		channelIcon.image = nil;
	}
	
	channelNumber.text = channum;
	
	if ([program.subtitle length] > 0 && [program.description length] > 0) {
		subtitleText.text = [NSString stringWithFormat:@"%@:\n%@", program.subtitle, program.description];
	}
	else if ([program.subtitle length] > 0 || [program.description length] > 0) {
		subtitleText.text = [NSString stringWithFormat:@"%@", ([program.subtitle length] > 0 ? program.subtitle : program.description)];
	}
	else {
		subtitleText.text = @"";
	}
	[subtitleText scrollRectToVisible:CGRectMake(0, 0, 1, 1) animated:NO];
	
	if (subtitleText.contentSize.height > subtitleText.bounds.size.height) {
		[subtitleText flashScrollIndicators];
	}
}

- (void)goToChannel:(id)sender {
	[TELNETREMOTE queryLocationWithTarget:self selector:@selector(locationResult:)];
}

- (void)locationResult:(NSString *)result {
	if (!result) {
		[[[[UIAlertView alloc] initWithTitle:@"Mythtv Location Query Failed" message:nil delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil] autorelease] show];
	}
	else {
		if ([result hasPrefix:@"Playback LiveTV"]) {
			[TELNETREMOTE sendCommand:[NSString stringWithFormat:@"play channel %@", channum]];
		}
		else {
			[[[[UIAlertView alloc] initWithTitle:@"Switch To Live TV" message:@"Mythtv is not watching live TV. Do you want to switch to live TV?" delegate:self cancelButtonTitle:@"Yes" otherButtonTitles:@"No", nil] autorelease] show];
		}
	}
}

- (void)dealloc
{
	[program release];
	[channelIcon release];
	[channelCallSign release];
	[channelNumber release];
	[titleText release];
	[metaText release];
	[subtitleText release];
	[super dealloc];
}

// UIAlertViewDelegate

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {
	if (buttonIndex == 0) {
		[TELNETREMOTE sendCommand:[NSString stringWithFormat:@"jump livetv\nplay channel %@", channum]];
	}
}

@end