//
//  RecordedEpisode.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "BackgroundView.h"
#import "RecordedEpisode.h"
#import "RecordedEpisodes.h"

@implementation RecordedEpisode

@synthesize episode;

- (id)initWithEpisode:(NSDictionary *)newEpisode {
	if (self = [super init]) {
		episode = [newEpisode retain];
		
		self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemPlay target:self action:@selector(playEpisode)] autorelease];
	}
	return self;
}

#define TITLE_HEIGHT 100
#define PREVIEW_WIDTH 124 // 70 * 16/9

- (void)loadView {
	self.view = [[[UIView alloc] initWithFrame:[UIScreen mainScreen].applicationFrame] autorelease];
	self.view.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
	self.view.autoresizesSubviews = YES;
	self.view.backgroundColor = [UIColor groupTableViewBackgroundColor];
	
	UIImage *preview = [episode objectForKey:@"preview"];
	imageView = [[[UIImageView alloc] init] autorelease];
	imageView.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin;
	imageView.backgroundColor = [UIColor clearColor];
	if (preview != (UIImage *)[NSNull null]) {
		imageView.frame = CGRectMake(10 + (PREVIEW_WIDTH - preview.size.width / SCALE) / 2, 10 + (TITLE_HEIGHT - preview.size.height / SCALE) / 2, preview.size.width / SCALE, preview.size.height / SCALE);
		imageView.image = preview;
	}
	[self.view addSubview:imageView];
	
	UILabel *label = [[[UILabel alloc] initWithFrame:CGRectMake(10 + PREVIEW_WIDTH + 10, 10, self.view.bounds.size.width - 10 - PREVIEW_WIDTH - 10 - 10, TITLE_HEIGHT)] autorelease];
	label.autoresizingMask = UIViewAutoresizingFlexibleWidth;
	label.backgroundColor = [UIColor clearColor];
	label.text = [episode objectForKey:@"title"];
	label.font = [UIFont boldSystemFontOfSize:20];
	label.textAlignment = UITextAlignmentCenter;
	label.numberOfLines = 0;
	label.lineBreakMode = UILineBreakModeWordWrap;
	[self.view addSubview:label];
		
	UITableView *tableView = [[[UITableView alloc] initWithFrame:CGRectMake(0, 10 + TITLE_HEIGHT, self.view.bounds.size.width, self.view.bounds.size.height - 10 - TITLE_HEIGHT - 10) style:UITableViewStyleGrouped] autorelease];
	tableView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
	tableView.dataSource = self;
	tableView.delegate = self;
	[self.view addSubview:tableView];
}

- (void)updatePreview {
	UIImage *preview = [episode objectForKey:@"preview"];
	imageView.frame = CGRectMake(10 + (PREVIEW_WIDTH - preview.size.width) / 2, 10 + (TITLE_HEIGHT - preview.size.height) / 2, preview.size.width, preview.size.height);
	imageView.image = preview;
}

- (void)playEpisode {
	RecordedEpisodes *recordedEpisodes = [self.navigationController.viewControllers objectAtIndex:(self.navigationController.viewControllers.count - 2)];
	[recordedEpisodes playEpisode:episode];
}

- (void)dealloc {
	[episode release];
	[super dealloc];
}

// UITableViewDataSource

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	NSInteger rows = 2;
	
	if ([[episode objectForKey:@"subTitle"] length]) {
		rows++;
	}
	if ([[episode objectForKey:@"airdate"] length]) {
		rows++;
	}
	if ([[episode objectForKey:@"category"] length]) {
		rows++;
	}
	if ([[episode objectForKey:@"Description"] length]) {
		rows++;
	}
	
	return rows;
}

#define SUBTITLE 0
#define START_END 1
#define AIRDATE 2
#define CHANNEL 3
#define CATEGORY 4
#define DESCRIPTION 5

- (NSInteger)dataIndexForRow:(NSUInteger)row {
	NSInteger dataIndex = 0;

	if (![[episode objectForKey:@"subTitle"] length]) {
		dataIndex++;
	}
	
	for (; row > 0; row--) {
		switch (dataIndex) {
			case SUBTITLE:
				if ([[episode objectForKey:@"subTitle"] length]) {
					dataIndex++;
					break;
				}
				
			case START_END:
				dataIndex++;
				break;
				
			case AIRDATE:
				if ([[episode objectForKey:@"airdate"] length]) {
					dataIndex++;
					break;
				}
				
			case CHANNEL:
				dataIndex++;
				break;
				
			case CATEGORY:
				if ([[episode objectForKey:@"category"] length]) {
					dataIndex++;
					break;
				}
		}
	}
	
	return dataIndex;
}

- (NSString *)valueForDataIndex:(NSInteger)dataIndex {
	NSDateFormatter *formatter;
	NSString *start;
	NSString *end;
	
	switch (dataIndex) {
		case SUBTITLE:
			return [episode objectForKey:@"subTitle"];
			
		case START_END:
			formatter = [[[NSDateFormatter alloc] init] autorelease];
			formatter.dateStyle = NSDateFormatterLongStyle;
			
			formatter.timeStyle = NSDateFormatterShortStyle;	
			start = [formatter stringFromDate:[episode objectForKey:@"startDate"]];
			
			formatter.dateStyle = NSDateFormatterNoStyle;
			end = [formatter stringFromDate:[episode objectForKey:@"endDate"]];
			
			return [NSString stringWithFormat:@"%@ - %@", start, end];
			
		case AIRDATE:
			formatter = [[[NSDateFormatter alloc] init] autorelease];
			formatter.dateStyle = NSDateFormatterLongStyle;
			
			return [formatter stringFromDate:[episode objectForKey:@"airDate"]];
			
		case CHANNEL:
			return [episode objectForKey:@"channelName"];
			
		case CATEGORY:
			return [episode objectForKey:@"category"];
			
		case DESCRIPTION:
			return [episode objectForKey:@"Description"];
			
		default:
			return @"";
	}
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	UITableViewCell *cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue2 reuseIdentifier:nil] autorelease];
	cell.selectionStyle = UITableViewCellSelectionStyleNone;

	NSInteger dataIndex = [self dataIndexForRow:indexPath.row];	

	switch (dataIndex) {
		case SUBTITLE:
			cell.textLabel.text = @"Subtitle:";
			break;
			
		case START_END:
			cell.textLabel.text = @"Time:";
			break;
			
		case AIRDATE:
			cell.textLabel.text = @"Originally:";
			break;
			
		case CHANNEL:
			cell.textLabel.text = @"Channel:";
			break;
			
		case CATEGORY:
			cell.textLabel.text = @"Category:";
			break;
			
		case DESCRIPTION:
			cell.textLabel.text = @"Desc:";
			break;
	}
	
	UILabel *detailTextLabel = [[UILabel alloc] initWithFrame:CGRectMake(83, 12, 207, 0)];
	detailTextLabel.font = [UIFont boldSystemFontOfSize:15];
	detailTextLabel.text = [self valueForDataIndex:dataIndex];
	detailTextLabel.numberOfLines = 0;
	CGRect frame = detailTextLabel.frame;
	frame.size = [detailTextLabel.text sizeWithFont:[UIFont boldSystemFontOfSize:15] constrainedToSize:CGSizeMake(207, INT_MAX) lineBreakMode:UILineBreakModeWordWrap];
	detailTextLabel.frame = frame;
	[cell.contentView addSubview:detailTextLabel];
	[detailTextLabel release];	
	
	return cell;
}

// UITableViewDelegate

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
	NSString *value = [self valueForDataIndex:[self dataIndexForRow:indexPath.row]];
	CGSize size = [value sizeWithFont:[UIFont boldSystemFontOfSize:15] constrainedToSize:CGSizeMake(207, INT_MAX) lineBreakMode:UILineBreakModeWordWrap];
	
	return MAX(35, size.height + 20);
}

@end
