//
//  RecordedEpisodes.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "defines.h"
#import "MainTabBarController.h"
#import "RecordedEpisode.h"
#import "RecordedEpisodes.h"
#import "Remote.h"
#import "TelnetRemote.h"

@implementation RecordedEpisodes

- (id)initWithProgram:(NSString *)name episodes:(NSArray *)newEpisodes {
	if (self = [super initWithStyle:UITableViewStylePlain]) {
		self.title = name;
		
		self.tableView.rowHeight = 75;
		
		episodes = [newEpisodes retain];
	}
	return self;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	return episodes.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"Episodes"];
	if (cell == nil) {
		cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"Episodes"] autorelease];
	}
	
	NSMutableDictionary *episode = [episodes objectAtIndex:indexPath.row];
	
	cell.textLabel.text = [[episodes objectAtIndex:indexPath.row] objectForKey:@"subTitle"];
	if (!cell.textLabel.text || cell.textLabel.text.length <= 0) {
		NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
		formatter.dateStyle = NSDateFormatterMediumStyle;
		formatter.timeStyle = NSDateFormatterShortStyle;
		cell.textLabel.text = [formatter stringFromDate:[episode objectForKey:@"startDate"]];
		[formatter release];
	}
	
	cell.accessoryType = UITableViewCellAccessoryDetailDisclosureButton;
	
	UIImage *preview = [episode objectForKey:@"preview"];
	if (!preview) {
		[episode setObject:[NSNull null] forKey:@"preview"];
		cell.imageView.image = nil;
		NSInvocationOperation *operation = [[NSInvocationOperation alloc] initWithTarget:self selector:@selector(retrievePreviewIconForEpisode:) object:[[NSNumber alloc] initWithUnsignedInteger:indexPath.row]];
		[OPERATION_QUEUE addOperation:operation];
		[operation release];
	}
	else if (preview != (UIImage *)[NSNull null]) {
		cell.imageView.image = preview;
	}
	else {
		cell.imageView.image = nil;
	}
	
	return cell;
}
																				
- (void)retrievePreviewIconForEpisode:(NSNumber *)episodeNumber {
	NSMutableDictionary *episode = [episodes objectAtIndex:episodeNumber.unsignedIntegerValue];
	[episodeNumber release];
	
	NSURLResponse *iconResponse;
	NSData *iconData = [NSURLConnection sendSynchronousRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:[NSString stringWithFormat:@"http://%@/Myth/GetPreviewImage?ChanId=%@&StartTime=%@&Height=%d", BACKEND_LOCATION, [episode objectForKey:@"chanId"], [episode objectForKey:@"recStartTs"], (int)(70 * SCALE)]]] returningResponse:&iconResponse error:NULL];
	UIImage *icon = [UIImage imageWithData:iconData];
	if (icon) {
		[episode setObject:icon forKey:@"preview"];
		UITableViewCell *cell = [self.tableView cellForRowAtIndexPath:[NSIndexPath indexPathForRow:episodeNumber.unsignedIntegerValue inSection:0]];
		cell.imageView.image = icon;
		[cell setNeedsLayout];
		
		if ([self.navigationController.topViewController isKindOfClass:[RecordedEpisode class]]) {
			RecordedEpisode *recordedEpisode = (RecordedEpisode *)self.navigationController.topViewController;
			if (recordedEpisode.episode == episode) {
				[recordedEpisode updatePreview];
			}
		}
	}
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
	[tableView deselectRowAtIndexPath:indexPath animated:YES];
	[self playEpisode:[episodes objectAtIndex:indexPath.row]];
}

- (void)tableView:(UITableView *)tableView accessoryButtonTappedForRowWithIndexPath:(NSIndexPath *)indexPath {
	[self.navigationController pushViewController:[[[RecordedEpisode alloc] initWithEpisode:[episodes objectAtIndex:indexPath.row]] autorelease] animated:YES];	
}
	
- (void)playEpisode:(NSDictionary *)episode {
	episodeToPlay = [episode retain];
	[[[[UIAlertView alloc] initWithTitle:@"Play Recording?" message:@"Do you want to begin playing this recorded episode?" delegate:self cancelButtonTitle:@"Yes" otherButtonTitles:@"No", nil] autorelease] show];
}

- (void)alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex {
	if (buttonIndex == 0) {
		[TELNETREMOTE sendCommand:[NSString stringWithFormat:@"play program %@ %@ resume", [episodeToPlay objectForKey:@"chanId"], [episodeToPlay objectForKey:@"recStartTs"]]];
		for (UIViewController *viewController in TAB_BAR_CONTROLLER.viewControllers) {
			if ([viewController isMemberOfClass:[Remote class]]) {
				TAB_BAR_CONTROLLER.selectedViewController = viewController;
				break;
			}
		}
	}
	[episodeToPlay release];
}

- (void)dealloc {
	[episodes release];
	[super dealloc];
}

@end
