//
//  RecordedPrograms.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "RecordedEpisodes.h"
#import "RecordedPrograms.h"
#import "Recordings.h"
#import "RecordingsData.h"

@implementation RecordedPrograms

- (id)initWithRecordings:(Recordings *)newRecordings programs:(NSMutableDictionary *)newPrograms {
	if (self = [super initWithStyle:UITableViewStylePlain]) {
		self.title = @"Recordings";
		
		UIActivityIndicatorView *activityIndicator = [[[UIActivityIndicatorView alloc] initWithFrame:CGRectMake(0, 0, 20, 20)] autorelease];
		[activityIndicator startAnimating];
		activityItem = [[UIBarButtonItem alloc] initWithCustomView:activityIndicator];
		self.navigationItem.rightBarButtonItem = activityItem;
		
		refreshItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh target:self action:@selector(refresh)];
		
		recordings = [newRecordings retain];
		programs = [newPrograms retain];
	}
	return self;
}

- (void)refresh {
	[self.navigationItem setRightBarButtonItem:activityItem animated:YES];
	[programs removeAllObjects];
	[self.tableView reloadData];
	[recordings refresh];
}

- (void)dataRefreshed {
	[self.navigationItem setRightBarButtonItem:refreshItem animated:YES];
	[self.tableView reloadData];
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	[sortedTitles release];
	sortedTitles = [[[programs allKeys] sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)] retain];
	return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	return [sortedTitles count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"Program"];
	if (cell == nil) {
		cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"Program"] autorelease];
	}
	
	cell.textLabel.text = [sortedTitles objectAtIndex:indexPath.row];
	cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
	
	return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
	[tableView deselectRowAtIndexPath:indexPath animated:YES];
	UITableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
	[self.navigationController pushViewController:[[[RecordedEpisodes alloc] initWithProgram:cell.textLabel.text episodes:[programs objectForKey:cell.textLabel.text]] autorelease] animated:YES];
}

- (void)dealloc {
	[programs release];
	[sortedTitles release];
	[recordings release];
	[activityItem release];
	[refreshItem release];
	[super dealloc];
}

@end
