//
//  RecordingsData.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "MythXMLParser.h"
#import "Recordings.h"
#import "RecordingsData.h"

@implementation RecordingsData

@synthesize programs;

- (id)initWithRecordings:(Recordings *)newRecordings {
	if (self = [super init]) {
		recordings = newRecordings;
		programs = [[NSMutableDictionary alloc] init];
	}
	return self;
}

- (void)refresh {
	[programs removeAllObjects];
	MythXMLParser *parser = [[MythXMLParser alloc] initWithContentsOfURL:[NSURL URLWithString:[NSString stringWithFormat:@"http://%@/Myth/GetRecorded", BACKEND_LOCATION]]];
	parser.delegate = self;
	if ([parser parse] == NO) {
		[[[[UIAlertView alloc] initWithTitle:@"Could Not Retrieving Recordings" message:@"Please verify that the MythTV backend is running." delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil] autorelease] show];
		NSLog(@"Error Retrieving Recordings: %@", [[parser parserError] localizedDescription]);
		NSLog(@"URL Used: http://%@/Myth/GetRecorded", BACKEND_LOCATION);
	}
	
	[parser release];	
	[recordings dataRefreshed];
}

- (void)dealloc {
	[programs release];
	[currentAttributes release];
	[currentDescription release];
	[super dealloc];
}

// NSXMLParser Delegate

- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qualifiedName attributes:(NSDictionary *)attributeDict {
	if ([elementName isEqualToString:@"Program"]) {
		currentDescription = [[NSMutableString alloc] init];
		currentAttributes = [attributeDict retain];
	}
	else if ([elementName isEqualToString:@"Channel"]) {
		[currentAttributes addEntriesFromDictionary:attributeDict];
	}
	else if ([elementName isEqualToString:@"Recording"]) {
		if ([[attributeDict objectForKey:@"recGroup"] isEqualToString:@"LiveTV"]) {
			[currentDescription release];
			currentDescription = nil;
			[currentAttributes release];
			currentAttributes = nil;
		}
		else {
			[currentAttributes addEntriesFromDictionary:attributeDict];
			NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
			if ([[currentAttributes objectForKey:@"airdate"] length]) {
				formatter.dateFormat = @"yyyy-MM-dd";
				[currentAttributes setObject:[formatter dateFromString:[currentAttributes objectForKey:@"airdate"]] forKey:@"airDate"];
			}
			formatter.dateFormat = @"yyyy-MM-dd'T'HH:mm:ss";
			[currentAttributes setObject:[formatter dateFromString:[currentAttributes objectForKey:@"recStartTs"]] forKey:@"startDate"];
			[currentAttributes setObject:[formatter dateFromString:[currentAttributes objectForKey:@"recEndTs"]] forKey:@"endDate"];
			[formatter release];
		}
	}
}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string {
	[currentDescription appendString:string];
}

- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qualifiedName {
	if ([elementName isEqualToString:@"Program"] && currentDescription && currentAttributes) {
		NSMutableDictionary *attributeDict = [NSMutableDictionary dictionaryWithDictionary:currentAttributes];
		[currentAttributes release];
		currentAttributes = nil;
		
		NSString *description = [currentDescription stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
		[currentDescription release];
		currentDescription = nil;

		if ([description length] > 0) {
			[attributeDict setObject:description forKey:@"Description"];
		}
		
		if (![programs objectForKey:[attributeDict objectForKey:@"title"]]) {
			[programs setObject:[NSMutableArray arrayWithObject:attributeDict] forKey:[attributeDict objectForKey:@"title"]];
		}
		else {
			[[programs objectForKey:[attributeDict objectForKey:@"title"]] addObject:attributeDict];
		}
	}
}

- (void)parser:(NSXMLParser *)parser parseErrorOccurred:(NSError *)parseError {
	[currentDescription release];
	currentDescription = nil;
	[currentAttributes release];
	currentAttributes = nil;
}

@end
