//
//  Remote.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import <libkern/OSAtomic.h>

#import "defines.h"
#import "ButtonPickerController.h"
#import "MainTabBarController.h"
#import "Remote.h"
#import "RemoteViewController.h"
#import "TelnetRemote.h"

@implementation Remote

@synthesize buttonPickerController;

- (id)init {
	if (self = [super init]) {
		self.title = @"Remote";
		self.tabBarItem.image = [UIImage imageNamed:@"remote.png"];
		
		buttonPickerController = [[ButtonPickerController alloc] init];
		
		remoteViewControllers = [[NSMutableDictionary alloc] init];
		for (NSString *remoteType in [NSArray arrayWithObjects:@"Menu", @"Live TV", @"Recording Playback", @"Video", @"Music", @"Weather", @"Numbers", @"Navigation", nil]) {
			RemoteViewController *remoteViewController = [[RemoteViewController alloc] initWithType:remoteType parent:self];
			[remoteViewControllers setObject:remoteViewController forKey:remoteType];
			[remoteViewController release];
		}

		navigationController = [[UINavigationController alloc] initWithRootViewController:[remoteViewControllers objectForKey:@"Menu"]];
		navigationController.navigationBar.barStyle = UIBarStyleBlackOpaque;
		self.view = navigationController.view;

		// Fix for iPhone OS 3.0 Beta
		CGRect frame = navigationController.navigationBar.frame;
		frame.origin.y = 0;
		navigationController.navigationBar.frame = frame;
	}
	return self;
}

- (void)viewWillDisappear:(BOOL)animated {
	[navigationController.topViewController setEditing:NO animated:YES];
}

- (void)showMusic {
	[navigationController pushViewController:[remoteViewControllers objectForKey:@"Music"] animated:YES];
}

- (void)showNumbers {
	[navigationController pushViewController:[remoteViewControllers objectForKey:@"Numbers"] animated:YES];
}

- (void)showNavigation {
	[navigationController pushViewController:[remoteViewControllers objectForKey:@"Navigation"] animated:YES];
}

- (void)showKeyboard {
	[HIDDEN_TEXT_FIELD becomeFirstResponder];
}

- (void)startPolling {
	if (!timer) {
		timer = [NSTimer scheduledTimerWithTimeInterval:3 target:self selector:@selector(queryLocation:) userInfo:nil repeats:YES];
	}
	[self queryLocation:nil];
}

- (void)stopPolling {
	[timer invalidate];
	timer = nil;
}

- (void)queryLocation:(NSTimer *)timer {
	if (!OSAtomicTestAndSet(0, &querying)) { // Make sure we don't have more than one query in flight at a time
		[TELNETREMOTE queryLocationWithTarget:self selector:@selector(updateLocation:)];
	}
}

- (void)updateLocation:(NSString *)locationString {
	if (!locationString) {
		querying = 0;
		return;
	}
	
	NSArray *location = [[locationString substringToIndex:([locationString length] - 2)] componentsSeparatedByCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
	NSString *newType;
	
	if ([[location objectAtIndex:0] isEqualToString:@"Playback"]) {
		if ([[location objectAtIndex:1] isEqualToString:@"LiveTV"]) {
			newType = @"Live TV";
		}
		else if ([[location objectAtIndex:1] isEqualToString:@"Recorded"]) {
			newType = @"Recording Playback";
		}
		else {
			newType = @"Video";
		}
	}
	else if ([[location objectAtIndex:0] isEqualToString:@"mythweather"]) {
		newType = @"Weather";
	}
	else {
		newType = @"Menu";
	}
	
	if (!currentType) {
		currentType = [newType retain];
		[navigationController setViewControllers:[NSArray arrayWithObject:[remoteViewControllers objectForKey:currentType]] animated:YES];
	}
	else if (![currentType isEqualToString:newType]) {
		[currentType release];
		currentType = [newType retain];
		[navigationController setViewControllers:[NSArray arrayWithObject:[remoteViewControllers objectForKey:currentType]] animated:YES];
	}
	
	querying = 0;
}

- (void)reloadRemoteViewController:(NSString *)type {
	[[remoteViewControllers objectForKey:type] loadView];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

- (void)dealloc {
	[currentType release];
	[remoteViewControllers release];
	[buttonPickerController release];
	[navigationController release];
	[super dealloc];
}

@end