//
//  RemoteButton.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "defines.h"
#import "ButtonPickerController.h"
#import "Remote.h"
#import "RemoteButton.h"
#import "RemoteView.h"
#import "RemoteViewController.h"
#import "TelnetRemote.h"

@implementation RemoteButton

@synthesize buttonPickerController;
@synthesize editing;
@synthesize info;
@synthesize targetView;

#define X [[[info objectForKey:@"Center"] objectAtIndex:0] floatValue]
#define Y [[[info objectForKey:@"Center"] objectAtIndex:1] floatValue]
#define X_SIZE [[[info objectForKey:@"Size"] objectAtIndex:0] floatValue]
#define Y_SIZE [[[info objectForKey:@"Size"] objectAtIndex:1] floatValue]

- (id)initWithInfo:(NSDictionary *)newInfo {
	if (self = [super init]) {
		info = [[NSMutableDictionary alloc] initWithDictionary:newInfo];
		
		self.backgroundColor = [UIColor clearColor];
		if ([info objectForKey:@"Image"]) {
			image = [[UIImage imageNamed:[NSString stringWithFormat:@"%@.png", [info objectForKey:@"Image"]]] retain];
		}
		
		self.frame = CGRectMake(0, 0, X_SIZE, Y_SIZE);
		self.center = CGPointMake(X, Y);		
	}
	return self;
}

- (void)drawRect:(CGRect)rect {
	CGFloat width = self.frame.size.width;
	
	CGContextRef ctx = UIGraphicsGetCurrentContext();
	
	CGContextMoveToPoint(ctx, 4 + width / 6, 4 + width / 6);
	CGContextAddArc(ctx, 4 + width / 6, 4 + width / 6, width / 6, M_PI, 3 * M_PI_2, 0);
	CGContextMoveToPoint(ctx, width - (4 + width / 6), 4 + width / 6);
	CGContextAddArc(ctx, width - (4 + width / 6), 4 + width / 6, width / 6, 3 * M_PI_2, 2 * M_PI, 0);
	CGContextMoveToPoint(ctx, 4 + width / 6, width - (4 + width / 6));
	CGContextAddArc(ctx, 4 + width / 6, width - (4 + width / 6), width / 6, M_PI_2, M_PI, 0);
	CGContextMoveToPoint(ctx, width - (4 + width / 6), width - (4 + width / 6));
	CGContextAddArc(ctx, width - (4 + width / 6), width - (4 + width / 6), width / 6, 0, M_PI_2, 0);
	CGContextAddRect(ctx, CGRectMake(4 + width / 6, 4, width - 2 * (4 + width / 6), width - 8));
	CGContextAddRect(ctx, CGRectMake(4, 4 + width / 6, width / 6, width - 2 * (4 + width / 6)));
	CGContextAddRect(ctx, CGRectMake(width - (4 + width / 6), 4 + width / 6, width / 6, width - 2 * (4 + width / 6)));
	CGContextClip(ctx);
	
	CGContextAddRect(ctx, rect);
	CGContextSetRGBFillColor(ctx, 0, 0, 0, 1);
	CGContextFillPath(ctx);
	
	if (image) {
		[image drawInRect:CGRectMake(4, 4, width - 8, width - 8)];
	}
	else {
		CGContextSetRGBFillColor(ctx, 1, 1, 1, 1);
		[[info objectForKey:@"Title"] drawCenteredInRect:CGRectMake(4, 4, width - 8, width - 8) withFont:[UIFont boldSystemFontOfSize:MAX(width / 5, 12)]];
	}
	
	CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
	
	CGFloat colors[8] = {1, 1, 1, 0.65, 1, 1, 1, 0.15};
	if ((self.tracking && self.highlighted) || editing) {
		colors[3] = 0.175;
		colors[7] = 0.05;
	}
	CGFloat locations[2] = {0, 1};
	CGGradientRef topGradient = CGGradientCreateWithColorComponents(colorSpace, colors, locations, 2);
	CGContextBeginPath(ctx);
	CGContextAddRect(ctx, CGRectMake(4, 4, width - 8, width / 2));
	CGContextDrawLinearGradient(ctx, topGradient, CGPointMake(0, 4), CGPointMake(0, width / 2), 0);
	CFRelease(topGradient);
	
	colors[3] = 0;
	if ((!self.tracking || !self.highlighted) && !editing) {
		colors[7] = 0.3;
	}
	else {
		colors[7] = 0.06;
	}
	CGGradientRef bottomGradient = CGGradientCreateWithColorComponents(colorSpace, colors, locations, 2);
	CGContextBeginPath(ctx);
	CGContextAddRect(ctx, CGRectMake(4, width - (4 + width / 6), width - 8, width / 6));
	CGContextDrawLinearGradient(ctx, bottomGradient, CGPointMake(0, width - (4 + width / 6)), CGPointMake(0, width - 4), 0);
	CFRelease(bottomGradient);
	
	CGColorSpaceRelease(colorSpace);

	previouslyHighlighted = self.highlighted;
}

- (void)setEditing:(BOOL)newEditing {
	editing = newEditing;
	[self setNeedsDisplay];
}

- (BOOL)beginTrackingWithTouch:(UITouch *)touch withEvent:(UIEvent *)event {
	[self setNeedsDisplay];
	return YES;
}

- (BOOL)continueTrackingWithTouch:(UITouch *)touch withEvent:(UIEvent *)event {
	if (previouslyHighlighted != self.highlighted) {
		[self setNeedsDisplay];
	}
	return YES;
}

- (void)endTrackingWithTouch:(UITouch *)touch withEvent:(UIEvent *)event {
	[self setNeedsDisplay];
}

#define INITIAL_TIMER 0
#define REPEAT_TIMER 1

- (void)prepareRepeatable {
	[self sendKey];
	[self removeTarget:self action:NULL forControlEvents:UIControlEventAllEvents];
	[self addTarget:self action:@selector(endRepeatable) forControlEvents:(UIControlEventTouchUpInside | UIControlEventTouchUpOutside | UIControlEventTouchDragExit)];
	timer = [NSTimer scheduledTimerWithTimeInterval:[DEFAULTS doubleForKey:@"Initial Repeat Rate"] target:self selector:@selector(repeat:) userInfo:[NSNumber numberWithInt:INITIAL_TIMER] repeats:NO];
}

- (void)repeat:(NSTimer *)firedTimer {
	[self sendKey];
	if ([[firedTimer userInfo] intValue] == INITIAL_TIMER) {
		timer = [NSTimer scheduledTimerWithTimeInterval:[DEFAULTS doubleForKey:@"Repeat Rate"] target:self selector:@selector(repeat:) userInfo:[NSNumber numberWithInt:REPEAT_TIMER] repeats:YES];
	}
}

- (void)endRepeatable {
	[timer invalidate];
	[self removeTarget:self action:NULL forControlEvents:UIControlEventAllEvents];
	[self addTarget:self action:@selector(prepareRepeatable) forControlEvents:UIControlEventTouchDown];
}

- (void)sendKey {
	if ([[info objectForKey:@"Context"] isEqualToString:@"MyMote"]) {
		[self handleMyMoteKey];
	}
	else if ([[info objectForKey:@"Context"] isEqualToString:@"LIRC"]) {
		[self handleLIRCKey];
	}
	else {
		NSString *key = [[[[DEFAULTS objectForKey:@"Keys"] objectForKey:[info objectForKey:@"Context"]] objectForKey:[info objectForKey:@"Action"]] objectForKey:@"Key"];
		if (key) {
			[TELNETREMOTE sendCommand:[NSString stringWithFormat:@"key %@", key]];
		}
	}
}

- (void)handleMyMoteKey {
	if ([[info objectForKey:@"Action"] isEqualToString:@"Music"]) {
		[REMOTE showMusic];
	}
	if ([[info objectForKey:@"Action"] isEqualToString:@"Numbers"]) {
		[REMOTE showNumbers];
	}
	else if ([[info objectForKey:@"Action"] isEqualToString:@"Navigation"]) {
		[REMOTE showNavigation];
	}
	else if ([[info objectForKey:@"Action"] isEqualToString:@"Alphabet"]) {
		[REMOTE showKeyboard];
	}
}

- (void)handleLIRCKey {
	NSOutputStream *ostream;
	CFStreamCreatePairWithSocketToHost(kCFAllocatorDefault, (CFStringRef)FRONTEND_ADDRESS, 8765, NULL, (CFWriteStreamRef *)&ostream);
	[ostream scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
	[ostream open];
	NSString *command = [NSString stringWithFormat:@"simulate 0000000000000000 0 %@ remote\n", [info objectForKey:@"LIRC Button"]];
	[ostream write:(const uint8_t *)[command UTF8String] maxLength:[command length]];
	[ostream close];
	[ostream release];
}

- (void)buttonPicked {
	RemoteButton *newRemoteButton = [[RemoteButton alloc] initWithInfo:info];
	newRemoteButton.frame = [self.superview convertRect:self.frame toView:[UIApplication sharedApplication].keyWindow];
	buttonPickerController.pickedButton = newRemoteButton;
	newRemoteButton.targetView = (RemoteView *)buttonPickerController.targetController.view;
	[[buttonPickerController parentViewController] dismissModalViewControllerAnimated:YES];
	[[UIApplication sharedApplication].keyWindow addSubview:newRemoteButton];
	[newRemoteButton.targetView.remoteButtons addObject:newRemoteButton];
	
	[newRemoteButton addTarget:newRemoteButton action:@selector(remove) forControlEvents:UIControlEventTouchUpInside];
	newRemoteButton.userInteractionEnabled = NO;
}

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
	if (buttonPickerController) {
		if ([[info objectForKey:@"Context"] isEqualToString:@"LIRC"] && [[touches anyObject] tapCount] > 1) {
			[NSObject cancelPreviousPerformRequestsWithTarget:self];
		}
	}
	else {
		[super touchesBegan:touches withEvent:event];
	}
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
	if (buttonPickerController) {
		if (![[info objectForKey:@"Context"] isEqualToString:@"LIRC"]) {
			[self buttonPicked];
			return;
		}
	
		if ([[touches anyObject] tapCount] == 1) {
			[self performSelector:@selector(singleTap) withObject:nil afterDelay:0.25];
		}
		else {
			[buttonPickerController editLIRCButton:self];
		}
	}
	else {
		[super touchesEnded:touches withEvent:event];
	}
}

- (void)singleTap {
	[self buttonPicked];
}

- (void)pickerGone {
	self.frame = [self.superview convertRect:self.frame toView:targetView];
	[self removeFromSuperview];
	[targetView addSubview:self];
	[self autorelease];
	
	NSMutableDictionary *newLayouts = [NSMutableDictionary dictionaryWithDictionary:[DEFAULTS objectForKey:@"Remote Layouts"]];
	NSMutableArray *newButtonDescriptions = [NSMutableArray arrayWithArray:[newLayouts objectForKey:((RemoteView *)self.superview).remoteViewController.type]];
	[info setObject:[NSArray arrayWithObjects:[NSNumber numberWithFloat:self.center.x], [NSNumber numberWithFloat:self.center.y], nil] forKey:@"Center"];
	[newButtonDescriptions addObject:info];
	[newLayouts setObject:newButtonDescriptions forKey:((RemoteView *)self.superview).remoteViewController.type];
	[DEFAULTS setObject:newLayouts forKey:@"Remote Layouts"];
}

- (void)remove {
	if ([self.superview isMemberOfClass:[RemoteView class]]) { // This is a hack as this should never be false...
		NSMutableDictionary *newLayouts = [NSMutableDictionary dictionaryWithDictionary:[DEFAULTS objectForKey:@"Remote Layouts"]];
		NSMutableArray *newButtonDescriptions = [NSMutableArray arrayWithArray:[newLayouts objectForKey:((RemoteView *)self.superview).remoteViewController.type]];
		[newButtonDescriptions removeObject:info];
		[newLayouts setObject:newButtonDescriptions forKey:((RemoteView *)self.superview).remoteViewController.type];
		[DEFAULTS setObject:newLayouts forKey:@"Remote Layouts"];
		
		[self removeFromSuperview];
	}
}

- (void)dealloc {
	[image release];
	[info release];
	[super dealloc];
}

@end