//
//  RemoteView.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "defines.h"
#import "RemoteButton.h"
#import "RemoteView.h"
#import "RemoteViewController.h"

@implementation RemoteView

@synthesize remoteViewController;
@synthesize remoteButtons;
@synthesize remove;

- (id)initWithFrame:(CGRect)frame type:(NSString *)type {
    if (self = [super initWithFrame:frame]) {
		self.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
		self.multipleTouchEnabled = YES;
		
		remoteButtons = [[NSMutableArray alloc] init];
		NSArray *buttons = [[DEFAULTS objectForKey:@"Remote Layouts"] objectForKey:type];
		
		for (NSDictionary *info in buttons) {
			RemoteButton *remoteButton = [[RemoteButton alloc] initWithInfo:info];
			if ([[info objectForKey:@"Repeatable"] boolValue]) {
				[remoteButton addTarget:remoteButton action:@selector(prepareRepeatable) forControlEvents:UIControlEventTouchDown];
			}
			else {
				[remoteButton addTarget:remoteButton action:@selector(sendKey) forControlEvents:UIControlEventTouchUpInside];
			}
			[remoteButtons addObject:remoteButton];
			[self addSubview:remoteButton];
			[remoteButton release];
		}
    }
    return self;
}

- (void)setEditing:(BOOL)editing {
	for (RemoteButton *remoteButton in remoteButtons) {
		if (editing) {
			[remoteButton removeTarget:remoteButton action:NULL forControlEvents:UIControlEventAllEvents];
			[remoteButton addTarget:remoteButton action:@selector(remove) forControlEvents:UIControlEventTouchUpInside];
			remoteButton.userInteractionEnabled = NO;
		}
		else {
			[remoteButton removeTarget:remoteButton action:NULL forControlEvents:UIControlEventAllEvents];
			if ([[remoteButton.info objectForKey:@"Repeatable"] boolValue]) {
				[remoteButton addTarget:remoteButton action:@selector(prepareRepeatable) forControlEvents:UIControlEventTouchDown];
			}
			else {
				[remoteButton addTarget:remoteButton action:@selector(sendKey) forControlEvents:UIControlEventTouchUpInside];
			}
			remoteButton.userInteractionEnabled = YES;
		}
	}
}

- (void)setRemove:(BOOL)newRemove {
	remove = newRemove;
	
	for (RemoteButton *remoteButton in remoteButtons) {
		remoteButton.userInteractionEnabled = remove;
	}
}

- (void)updateEditedButtonForEvent:(UIEvent *)event {
	RemoteButton *oldButton = editingButton;
	
	UITouch *touch1 = nil;
	UITouch *touch2 = nil;
	for (UITouch *touch in [event allTouches]) {
		if (touch.phase != UITouchPhaseEnded && touch.phase != UITouchPhaseCancelled) {
			if (!touch1) {
				touch1 = touch;
			}
			else if (!touch2) {
				touch2 = touch;
			}
			else {
				touch1 = nil;
				touch2 = nil;
				break;
			}
		}
	}
	
	if (!touch1) {
		for (RemoteButton *remoteButton in self.subviews) {
			if (![remoteButton isMemberOfClass:[RemoteButton class]]) {
				continue;
			}
			remoteButton.editing = NO;
			editingButton = nil;
		}
		
		if (oldButton) {
			NSMutableDictionary *newLayouts = [NSMutableDictionary dictionaryWithDictionary:[DEFAULTS objectForKey:@"Remote Layouts"]];
			NSMutableArray *newButtonDescriptions = [NSMutableArray arrayWithArray:[newLayouts objectForKey:remoteViewController.type]];
			oldButton.center = CGPointMake(5 * (floorf((oldButton.center.x + 2.5) / 5)), 5 * (floorf((oldButton.center.y + 2.5) / 5)));
			oldButton.bounds = CGRectMake(oldButton.bounds.origin.x, oldButton.bounds.origin.y, 5 * (floorf((oldButton.bounds.size.width + 2.5) / 5)), 5 * (floorf((oldButton.bounds.size.height + 2.5) / 5)));
			[oldButton.info setObject:[NSArray arrayWithObjects:[NSNumber numberWithFloat:oldButton.center.x], [NSNumber numberWithFloat:oldButton.center.y], nil] forKey:@"Center"];
			[oldButton.info setObject:[NSArray arrayWithObjects:[NSNumber numberWithFloat:oldButton.bounds.size.width], [NSNumber numberWithFloat:oldButton.bounds.size.height], nil] forKey:@"Size"];
			[newButtonDescriptions replaceObjectAtIndex:editingButtonIndex withObject:oldButton.info];
			[newLayouts setObject:newButtonDescriptions forKey:remoteViewController.type];
			[DEFAULTS setObject:newLayouts forKey:@"Remote Layouts"];				
		}

		return;
	}
	
	CGPoint center;
	if (!touch2) {
		center = [touch1 locationInView:self];
	}
	else {
		center = CGPointMake([touch1 locationInView:self].x / 2 + [touch2 locationInView:self].x / 2, [touch1 locationInView:self].y / 2 + [touch2 locationInView:self].y / 2);
	}
	
	editingButton = nil;
	
	for (RemoteButton *remoteButton in [self.subviews reverseObjectEnumerator]) {
		if (![remoteButton isMemberOfClass:[RemoteButton class]]) {
			continue;
		}
		
		if (!editingButton && [remoteButton pointInside:[self convertPoint:center toView:remoteButton] withEvent:nil]) {
			remoteButton.editing = YES;
			[self bringSubviewToFront:remoteButton];
			editingButton = remoteButton;

			for (NSDictionary *buttonDescription in [[DEFAULTS objectForKey:@"Remote Layouts"] objectForKey:remoteViewController.type]) {
				if ([[[buttonDescription objectForKey:@"Center"] objectAtIndex:0] floatValue] == editingButton.center.x && [[[buttonDescription objectForKey:@"Center"] objectAtIndex:1] floatValue] == editingButton.center.y) {
					editingButtonIndex = [[[DEFAULTS objectForKey:@"Remote Layouts"] objectForKey:remoteViewController.type] indexOfObject:buttonDescription];
					break;
				}
			}
		}
		else {
			remoteButton.editing = NO;
		}
	}

	if (oldButton) {
		NSMutableDictionary *newLayouts = [NSMutableDictionary dictionaryWithDictionary:[DEFAULTS objectForKey:@"Remote Layouts"]];
		NSMutableArray *newButtonDescriptions = [NSMutableArray arrayWithArray:[newLayouts objectForKey:remoteViewController.type]];
		[oldButton.info setObject:[NSArray arrayWithObjects:[NSNumber numberWithFloat:oldButton.center.x], [NSNumber numberWithFloat:oldButton.center.y], nil] forKey:@"Center"];
		[oldButton.info setObject:[NSArray arrayWithObjects:[NSNumber numberWithFloat:oldButton.bounds.size.width], [NSNumber numberWithFloat:oldButton.bounds.size.height], nil] forKey:@"Size"];
		[newButtonDescriptions replaceObjectAtIndex:editingButtonIndex withObject:oldButton.info];
		[newLayouts setObject:newButtonDescriptions forKey:remoteViewController.type];
		[DEFAULTS setObject:newLayouts forKey:@"Remote Layouts"];				
	}	
}

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
	if (remoteViewController.editing && !remove) {
		[self updateEditedButtonForEvent:event];
	}
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
	if (!editingButton) {
		return;
	}
	
	if ([[event allTouches] count] == 1) {
		CGPoint movement;
		movement.x = [[touches anyObject] locationInView:self].x - [[touches anyObject] previousLocationInView:self].x;
		movement.y = [[touches anyObject] locationInView:self].y - [[touches anyObject] previousLocationInView:self].y;
		
		CGRect frame = editingButton.frame;
		frame.origin.x += movement.x;
		frame.origin.y += movement.y;
		editingButton.frame = frame;
	}
	else {
		NSArray *touchesArray = [[event allTouches] allObjects];
		CGPoint point1 = [[touchesArray objectAtIndex:0] locationInView:self];
		CGPoint previousPoint1 = [[touchesArray objectAtIndex:0] previousLocationInView:self];
		CGPoint point2 = [[touchesArray objectAtIndex:1] locationInView:self];
		CGPoint previousPoint2 = [[touchesArray objectAtIndex:1] previousLocationInView:self];
		
		CGPoint center = CGPointMake(point1.x / 2 + point2.x / 2, point1.y / 2 + point2.y / 2);
		CGPoint previousCenter = CGPointMake(previousPoint1.x / 2 + previousPoint2.x / 2, previousPoint1.y / 2 + previousPoint2.y / 2);
		
		CGFloat distance = sqrtf(powf(point2.x - point1.x, 2) + powf(point2.y - point1.y, 2));
		CGFloat previousDistance = sqrtf(powf(previousPoint2.x - previousPoint1.x, 2) + powf(previousPoint2.y - previousPoint1.y, 2));
		
		CGFloat difference = distance - previousDistance;
		
		CGRect frame;
		
		frame = editingButton.frame;
		
		if (frame.size.width + difference < 30) {
			difference = 30 - frame.size.width;
		}
		
		frame.origin.x += center.x - previousCenter.x - difference / 2;
		frame.origin.y += center.y - previousCenter.y - difference / 2;
		frame.size.width += difference;
		frame.size.height += difference;
		editingButton.frame = frame;
		
		[editingButton setNeedsDisplay];
	}
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
	[self updateEditedButtonForEvent:event];
}

- (void)touchesCanceled {
	editingButton = nil;
}

- (void)dealloc {
	for (RemoteButton *button in remoteButtons) {
		[button removeFromSuperview];
	}
	[remoteButtons release];
	[super dealloc];
}

@end
