//
//  RemoteNavController.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "ButtonPickerController.h"
#import "Remote.h"
#import "RemoteView.h"
#import "RemoteViewController.h"

@implementation RemoteViewController

@synthesize type;

- (id)initWithType:(NSString *)newType parent:(Remote *)newParent {
    if (self = [super init]) {
		type = newType;
		parent = newParent;
		
		self.title = type;
    }
    return self;
}

- (void)loadView {
	if (![type isEqualToString:@"Music"] && ![type isEqualToString:@"Numbers"] && ![type isEqualToString:@"Navigation"]) {
		self.navigationItem.hidesBackButton = YES;
	}
	self.navigationItem.rightBarButtonItem = [self editButtonItem];
	
	addRemove = [[UISegmentedControl alloc] initWithItems:[NSArray arrayWithObjects:@" - ", @" + ", nil]];
	addRemove.segmentedControlStyle = UISegmentedControlStyleBar;
	addRemove.tintColor = [UIColor grayColor];
	[addRemove addTarget:self action:@selector(addRemoveClicked) forControlEvents:UIControlEventValueChanged];
	addRemoveItem = [[UIBarButtonItem alloc] initWithCustomView:addRemove];
	
	RemoteView *remoteView = [[[RemoteView alloc] initWithFrame:[UIScreen mainScreen].applicationFrame type:type] autorelease];
	remoteView.remoteViewController = self;
	self.view = remoteView;
}

- (void)setEditing:(BOOL)editing animated:(BOOL)animated {
	[super setEditing:editing animated:animated];
	
	if (editing) {
		[self.navigationItem setLeftBarButtonItem:addRemoveItem animated:YES];
	}
	else {
		[self.navigationItem setLeftBarButtonItem:nil animated:YES];
		addRemove.selectedSegmentIndex = -1;
	}
	
	[(RemoteView *)self.view setEditing:editing];
}

- (void)addRemoveClicked {
	static int canAdd = 0;
	
	switch (addRemove.selectedSegmentIndex) {
		case 0:
			canAdd = -1;
			[(RemoteView *)self.view setRemove:YES];
			break;
			
		case 1:
			addRemove.selectedSegmentIndex = -1;
			if (canAdd == 0) {
				[self showPicker];
			}
			else {
				[(RemoteView *)self.view setRemove:NO];
				canAdd = 0;
			}
			break;
	}
}

- (void)showPicker {
	UINavigationController *modalController = [[[UINavigationController alloc] initWithRootViewController:parent.buttonPickerController] autorelease];
	parent.buttonPickerController.targetController = self;
	modalController.navigationBar.barStyle = UIBarStyleBlackOpaque;
	[self presentModalViewController:modalController animated:YES];
}

- (void)dealloc
{
	[addRemove release];
	[super dealloc];
}

@end
