//
//  RepeatRates.m
//  MyMote
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "RemoteButton.h"
#import "RepeatRates.h"

@implementation RepeatRates

#define INITIAL 0
#define REPEAT 1

#define INITIAL_DELAY_MIN 0.1
#define INITIAL_DELAY_MAX 1.9
#define REPEAT_RATE_MIN 0.1
#define REPEAT_RATE_MAX 0.9

- (id)init {
	if (self = [super init]) {
		self.title = @"Button Repeat Rate";
	}
	return self;
}

- (void)loadView {
	self.view = [[[UIView alloc] init] autorelease];
	self.view.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
	self.view.autoresizesSubviews = YES;
	self.view.backgroundColor = [UIColor groupTableViewBackgroundColor];
	
	UILabel *initialLabel = [[[UILabel alloc] init] autorelease];
	initialLabel.text = @"Initial Repeat Delay";
	initialLabel.font = [UIFont boldSystemFontOfSize:20];
	initialLabel.backgroundColor = [UIColor clearColor];
	[initialLabel sizeToFit];
	initialLabel.center = CGPointMake(160, 50);
	[self.view addSubview:initialLabel];
	
	UISlider *initialSlider = [[[UISlider alloc] initWithFrame:CGRectMake(0, 75, 0, 50)] autorelease];
	initialSlider.tag = INITIAL;
	initialSlider.autoresizingMask = UIViewAutoresizingFlexibleWidth;
	initialSlider.minimumValue = INITIAL_DELAY_MIN;
	initialSlider.maximumValue = INITIAL_DELAY_MAX;
	initialSlider.value = [DEFAULTS doubleForKey:@"Initial Repeat Rate"];
	initialSlider.continuous = YES;
	[self.view addSubview:initialSlider];

	UILabel *repeatLabel = [[[UILabel alloc] init] autorelease];
	repeatLabel.text = @"Repeat Delay";
	repeatLabel.font = [UIFont boldSystemFontOfSize:20];
	repeatLabel.backgroundColor = [UIColor clearColor];
	[repeatLabel sizeToFit];
	repeatLabel.center = CGPointMake(160, 150);
	[self.view addSubview:repeatLabel];
	
	UISlider *repeatSlider = [[[UISlider alloc] initWithFrame:CGRectMake(0, 175, 0, 50)] autorelease];
	repeatSlider.tag = REPEAT;
	repeatSlider.autoresizingMask = UIViewAutoresizingFlexibleWidth;
	repeatSlider.minimumValue = REPEAT_RATE_MIN;
	repeatSlider.maximumValue = REPEAT_RATE_MAX;
	repeatSlider.value = [DEFAULTS doubleForKey:@"Repeat Rate"];
	repeatSlider.continuous = YES;
	[self.view addSubview:repeatSlider];
	
	[initialSlider addTarget:self action:@selector(rateChanged:) forControlEvents:UIControlEventValueChanged];
	[repeatSlider addTarget:self action:@selector(rateChanged:) forControlEvents:UIControlEventValueChanged];
	
	UIView *indicatorBackgroundBorder = [[[UIView alloc] initWithFrame:CGRectMake(30, 225, 100, 100)] autorelease];
	indicatorBackgroundBorder.backgroundColor = [UIColor grayColor];
	[self.view addSubview:indicatorBackgroundBorder];
	
	UIView *indicatorBackground = [[[UIView alloc] initWithFrame:CGRectMake(5, 5, 90, 90)] autorelease];
	indicatorBackground.backgroundColor = [UIColor whiteColor];
	[indicatorBackgroundBorder addSubview:indicatorBackground];
	
	indicator = [[[UIView alloc] initWithFrame:CGRectMake(0, indicatorBackground.bounds.size.height, indicatorBackground.bounds.size.width, 0)] autorelease];
	indicator.backgroundColor = [UIColor blueColor];
	[indicatorBackground addSubview:indicator];
	
	NSDictionary *info = [NSDictionary dictionaryWithObjectsAndKeys:@"Test", @"Title", [NSArray arrayWithObjects:[NSNumber numberWithInt:240], [NSNumber numberWithInt:275], nil], @"Center", [NSArray arrayWithObjects:[NSNumber numberWithInt:100], [NSNumber numberWithInt:100], nil], @"Size", nil];
	testButton = [[[RemoteButton alloc] initWithInfo:info] autorelease];
	[self.view addSubview:testButton];
	
	[testButton addTarget:self action:@selector(prepareRepeatable) forControlEvents:UIControlEventTouchDown];
}

- (void)rateChanged:(UISlider *)slider {
	if (slider.tag == INITIAL) {
		[DEFAULTS setDouble:slider.value forKey:@"Initial Repeat Rate"];
	}
	else {
		[DEFAULTS setDouble:slider.value forKey:@"Repeat Rate"];
	}
}

- (void)indicate:(BOOL)initial {
	indicator.frame = indicator.superview.bounds;
	[UIView beginAnimations:nil context:NULL];
	if (initial) {
		[UIView setAnimationDuration:[DEFAULTS doubleForKey:@"Initial Repeat Rate"]];
	}
	else {
		[UIView setAnimationDuration:[DEFAULTS doubleForKey:@"Repeat Rate"]];
	}
	indicator.frame = CGRectMake(0, indicator.superview.bounds.size.height, indicator.superview.bounds.size.width, 0);
	[UIView commitAnimations];
}

#define INITIAL_TIMER 0
#define REPEAT_TIMER 1

- (void)prepareRepeatable {
	[self indicate:YES];
	[testButton removeTarget:self action:NULL forControlEvents:UIControlEventAllEvents];
	[testButton addTarget:self action:@selector(endRepeatable) forControlEvents:(UIControlEventTouchUpInside | UIControlEventTouchUpOutside | UIControlEventTouchDragExit)];
	timer = [NSTimer scheduledTimerWithTimeInterval:[DEFAULTS doubleForKey:@"Initial Repeat Rate"] target:self selector:@selector(repeat:) userInfo:[NSNumber numberWithInt:INITIAL_TIMER] repeats:NO];
}

- (void)repeat:(NSTimer *)firedTimer {
	[self indicate:NO];
	timer = [NSTimer scheduledTimerWithTimeInterval:[DEFAULTS doubleForKey:@"Repeat Rate"] target:self selector:@selector(repeat:) userInfo:[NSNumber numberWithInt:REPEAT_TIMER] repeats:NO];
}

- (void)endRepeatable {
	[timer invalidate];
	[testButton removeTarget:self action:NULL forControlEvents:UIControlEventAllEvents];
	[testButton addTarget:self action:@selector(prepareRepeatable) forControlEvents:UIControlEventTouchDown];
}

@end
