//
//  ResetRemoteViews.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "Remote.h"
#import "ResetRemoteViews.h"

@implementation ResetRemoteViews

- (id)init {
	if (self = [super initWithStyle:UITableViewStyleGrouped]) {
		self.title = @"Reset Remote Views";
		
		defaultLayouts = [[[NSDictionary dictionaryWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"Defaults" ofType:@"plist"]] objectForKey:@"Remote Layouts"] retain];
	}
	return self;
}


- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return 1;
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	return [defaultLayouts count] - 1;
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"ResetRemoteViewCell"];
	if (cell == nil) {
		cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"ResetRemoteViewCell"] autorelease];
	}
	
	cell.textLabel.text = [[defaultLayouts allKeys] objectAtIndex:indexPath.row];
	
	return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
	selectedIndexPath = [indexPath retain];
	NSString *remoteView = [self.tableView cellForRowAtIndexPath:indexPath].textLabel.text;
	[[[[UIAlertView alloc] initWithTitle:@"Confirm View Reset" message:[NSString stringWithFormat:@"Reset %@ Remote View?", remoteView] delegate:self cancelButtonTitle:@"No" otherButtonTitles:@"Yes", nil] autorelease] show];
}

- (void)dealloc {
	[selectedIndexPath release];
	[defaultLayouts release];
	[super dealloc];
}

// UIAlertViewDelegate

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {
	[alertView dismissWithClickedButtonIndex:buttonIndex animated:YES];
	
	if (buttonIndex == 1) {
		NSString *remoteView = [self.tableView cellForRowAtIndexPath:selectedIndexPath].textLabel.text;
		NSMutableDictionary *remoteLayouts = [NSMutableDictionary dictionaryWithDictionary:[DEFAULTS objectForKey:@"Remote Layouts"]];
		NSDictionary *layout = [defaultLayouts objectForKey:remoteView];
		[remoteLayouts setObject:layout forKey:remoteView];
		[DEFAULTS setObject:remoteLayouts forKey:@"Remote Layouts"];
		[REMOTE reloadRemoteViewController:remoteView];
	}
	
	[self.tableView deselectRowAtIndexPath:selectedIndexPath animated:YES];
	[selectedIndexPath release];
	selectedIndexPath = nil;
}

@end

