//
//  SettingsViewController.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import "defines.h"
#import "version.h"
#import "BackendSelect.h"
#import "FrontendSelect.h"
#import "RepeatRates.h"
#import "ResetRemoteViews.h"
#import "Settings.h"
#import "SettingsViewController.h"

@implementation SettingsViewController

- (id)init {
	if (self = [super initWithStyle:UITableViewStyleGrouped]) {
		self.title = @"Settings";
	}
	return self;
}

- (void)loadView {
	[super loadView];
	UIButton *info = [UIButton buttonWithType:UIButtonTypeInfoLight];
	[info addTarget:self action:@selector(info) forControlEvents:UIControlEventTouchUpInside];
	self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc] initWithCustomView:info] autorelease];
	activityView = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhite];
}

- (void)info {
	NSMutableString *message = [NSMutableString stringWithFormat:@"Author: Chase Douglas\nVersion: %@\nRevision: %d", [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleVersion"], MYMOTE_REVISION];
#if !MYMOTE_REVISION_CLEAN
	[message appendString:@"\nNot Clean"];
#endif
	if (DEBUG_INFO.length) {
		[message appendString:@"\n\nDebug Info:\n"];
		[message appendString:DEBUG_INFO];
	}
	[[[[UIAlertView alloc] initWithTitle:@"MyMote" message:message delegate:nil cancelButtonTitle:@"Done" otherButtonTitles:nil] autorelease] show];
}

- (void)disableAutoLock:(UISwitch *)sender {
	[[NSUserDefaults standardUserDefaults] setBool:sender.on forKey:@"Disable Auto-Lock"];
	[UIApplication sharedApplication].idleTimerDisabled = sender.on;
}

- (void)callPause:(UISwitch *)sender {
	[[NSUserDefaults standardUserDefaults] setBool:sender.on forKey:@"Pause When Called"];
}

- (void)dealloc {
	[activityView release];
	[super dealloc];
}

// UITableViewDataSource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return 5;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	switch (section) {
		case 0:
			return 2;
			
		case 1:
			return 1;
			
		case 2:
			return 1;
		
		case 3:
			return (NSClassFromString(@"CTCallCenter") ? 2 : 1);
		
		case 4:
			return 3;
	}
	
	return 0;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"settingsCell"];
	if (!cell) {
		cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:@"settingsCell"] autorelease];
	}
	
	cell.textLabel.text = nil;
	cell.detailTextLabel.text = nil;
	cell.accessoryType = UITableViewCellAccessoryNone;
	cell.selectionStyle = UITableViewCellSelectionStyleBlue;
	cell.accessoryView = nil;
	
	UISwitch *disableAutoLock;
	UISwitch *callPause;
	switch (indexPath.section) {
		case 0:
			switch (indexPath.row) {
				case 0:
					if (BACKEND.port == 6544) {
						cell.textLabel.text = @"Backend:";
						cell.detailTextLabel.text = BACKEND.hostname ? BACKEND.hostname : BACKEND.address;
					}
					else {
						cell.textLabel.text = @"Backend:";
						cell.detailTextLabel.text = [NSString stringWithFormat:@"%@:%hu", (BACKEND.hostname ? BACKEND.hostname : BACKEND.address), BACKEND.port];
					}
					break;
				
				case 1:
					cell.textLabel.text = @"Frontend:";
					cell.detailTextLabel.text = FRONTEND;
					break;
			}
			cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
			break;
			
		case 1:
			cell.textLabel.text = @"Button Repeat Rate";
			cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
			break;
			
		case 2:
			cell.textLabel.text = @"Reset Remote Layouts";
			cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
			break;
			
		case 3:
			switch (indexPath.row) {
				case 0:
					cell.textLabel.text = @"Disable Auto-Lock";
					cell.selectionStyle = UITableViewCellSelectionStyleNone;
					disableAutoLock = [[[UISwitch alloc] init] autorelease];
					disableAutoLock.on = [[NSUserDefaults standardUserDefaults] boolForKey:@"Disable Auto-Lock"];
					[disableAutoLock addTarget:self action:@selector(disableAutoLock:) forControlEvents:UIControlEventValueChanged];
					cell.accessoryView = disableAutoLock;
					break;
				case 1:
					cell.textLabel.text = @"Pause When Called";
					cell.selectionStyle = UITableViewCellSelectionStyleNone;
					callPause = [[[UISwitch alloc] init] autorelease];
					callPause.on = [[NSUserDefaults standardUserDefaults] boolForKey:@"Pause When Called"];
					[callPause addTarget:self action:@selector(callPause:) forControlEvents:UIControlEventValueChanged];
					cell.accessoryView = callPause;
			}
			break;
			
		case 4:
			switch (indexPath.row) {
				case 0:
					cell.textLabel.text = @"Manual";
					break;
				case 1:
					cell.textLabel.text = @"File A Bug";
					break;
				case 2:
					cell.textLabel.text = @"Donate";
					break;
			}
			break;
	}
	
	return cell;
}

// UITableViewDelegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
	switch (indexPath.section) {
		case 0:
			switch (indexPath.row) {
				case 0:
					[self.navigationController pushViewController:[[[BackendSelect alloc] init] autorelease] animated:YES];
					break;
			
				case 1:
					[self.navigationController pushViewController:[[[FrontendSelect alloc] initWithBackend:BACKEND] autorelease] animated:YES];
					break;
			}
			break;
			
		case 1:
			[self.navigationController pushViewController:[[[RepeatRates alloc] init] autorelease] animated:YES];
			break;
			
		case 2:
			[self.navigationController pushViewController:[[[ResetRemoteViews alloc] init] autorelease] animated:YES];
			break;
			
		case 4: {
			UIWebView *webView = [[UIWebView alloc] init];
			webView.scalesPageToFit = YES;
			webView.delegate = self;
			UIViewController *controller = [[UIViewController alloc] init];
			controller.view = webView;
			controller.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithCustomView:activityView];
			[controller.navigationItem.rightBarButtonItem release];
			[webView release];
			[self.navigationController pushViewController:controller animated:YES];
			[controller release];
			switch (indexPath.row) {
				case 0: {
					[webView loadRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:@"http://mymote.wikispot.org"]]];
					break;
				}
				case 1: {
					[webView loadRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:@"https://bugs.launchpad.net/mymote"]]];
					break;
				}
				case 2: {
					[webView loadRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:@"http://mymote.wikispot.org/Donations"]]];
					break;
				}
			}
			break;
		}
	}
	
	[tableView deselectRowAtIndexPath:indexPath animated:YES];
}

// UIWebViewDelegate

- (void)webViewDidStartLoad:(UIWebView *)webView {
	[activityView startAnimating];
}

- (void)webViewDidFinishLoad:(UIWebView *)webView {
	[activityView stopAnimating];
}

@end
