//
//  TelnetRemote.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import <arpa/inet.h>
#import <sys/socket.h>
#import <net/if.h>
#import <sys/ioctl.h>
#import <unistd.h>
#import <CFNetwork/CFNetwork.h>

#import "defines.h"
#import "BackendSelect.h"
#import "MainTabBarController.h"
#import "RemoteButton.h"
#import "Remote.h"
#import "Settings.h"
#import "TelnetRemote.h"

@implementation TelnetRemote

@synthesize resuming;

- (id)init {
	if (self = [super init]) {
		sendQueue = [[NSMutableArray alloc] init];
		recvQueue = [[NSMutableArray alloc] init];
		stateLock = [[NSLock alloc] init];
		message = [[NSMutableString alloc] init];
		toSend = [[NSMutableString alloc] init];
	}
	return self;
}

- (void)setup {
	if (([istream streamStatus] == NSStreamStatusOpen || [istream streamStatus] == NSStreamStatusReading) && ([ostream streamStatus] == NSStreamStatusOpen || [ostream streamStatus] == NSStreamStatusWriting)) {
		return;
	}
    
	if (istream || ostream) {
		[istream close];
		[ostream close];
		[istream removeFromRunLoop:runloop forMode:NSDefaultRunLoopMode];
		[ostream removeFromRunLoop:runloop forMode:NSDefaultRunLoopMode];
		[istream release];
		[ostream release];
		[stateLock lock];
		for (NSDictionary *command in sendQueue) {
			[[command objectForKey:@"Target"] performSelector:((SelectorObject *)[command objectForKey:@"Selector"]).selector withObject:nil];
		}
		[sendQueue release];
		sendQueue = [[NSMutableArray alloc] init];
		for (NSDictionary *command in recvQueue) {
			[[command objectForKey:@"Target"] performSelector:((SelectorObject *)[command objectForKey:@"Selector"]).selector withObject:nil];
		}
		[recvQueue release];
		recvQueue = [[NSMutableArray alloc] init];
		[toSend setString:@""];
		[stateLock unlock];
		istream = nil;
		ostream = nil;
	}
	
	struct if_nameindex *ifs = if_nameindex();
	int sock = socket(AF_INET, SOCK_STREAM, 0);
	struct if_nameindex *cur_if;
	for (cur_if = ifs; cur_if->if_index && cur_if->if_name; cur_if++) {
		struct ifreq req;
		strncpy(req.ifr_name, cur_if->if_name, IFNAMSIZ - 1);
		req.ifr_name[IFNAMSIZ - 1] = '\0';
		ioctl(sock, SIOCGIFFLAGS, &req);
		if (!(req.ifr_flags & IFF_UP)) {
			continue;
		}
		
		ioctl(sock, SIOCGIFADDR, &req);
		char addr[16];
		if (inet_ntop(AF_INET, &((struct sockaddr_in *)&req.ifr_addr)->sin_addr, addr, 16)) {
			if (![@"127.0.0.1" isEqualToString:[NSString stringWithFormat:@"%s", addr]]) {
				break;
			}
		}
	}
	close(sock);
		
	if (!cur_if->if_index || !cur_if->if_name) {
		if_freenameindex(ifs);
		if (!netDownAlert) {
			netDownAlert = [[UIAlertView alloc] initWithTitle:@"Network Is Down" message:@"There is no network connectivity. Please wait for connectivity or check the wireless network settings." delegate:nil cancelButtonTitle:nil otherButtonTitles:nil];
			[netDownAlert show];
		}
		[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(recheckNetConnectivity:) userInfo:nil repeats:NO];
		[REMOTE stopPolling];
		return;
	}
	else {
		if_freenameindex(ifs);
		if (netDownAlert) {
			[netDownAlert dismissWithClickedButtonIndex:0 animated:YES];
			[netDownAlert release];
			netDownAlert = nil;
		}
	}
	
	if (!connectingAlert && !TAB_BAR_CONTROLLER.modalViewController) {
		connectingAlert = [[[UIAlertView alloc] initWithTitle:@"Connecting..." message:nil delegate:self cancelButtonTitle:@"Select Frontend..." otherButtonTitles:nil] autorelease];
		[connectingAlert show];
	}
	
	CFStreamCreatePairWithSocketToHost(kCFAllocatorDefault, (CFStringRef)FRONTEND_ADDRESS, FRONTEND_PORT, (CFReadStreamRef *)&istream, (CFWriteStreamRef *)&ostream);
	
	istream.delegate = self;
	ostream.delegate = self;
	runloop = [NSRunLoop currentRunLoop];
	[istream scheduleInRunLoop:runloop forMode:NSDefaultRunLoopMode];
	[ostream scheduleInRunLoop:runloop forMode:NSDefaultRunLoopMode];
	[istream open];
	[ostream open];
	
	[REMOTE startPolling];
}

- (void)recheckNetConnectivity:(NSTimer *)timer {
	[self setup];
}

- (void)stream:(NSStream *)stream handleEvent:(NSStreamEvent)event {
	[connectingAlert dismissWithClickedButtonIndex:0 animated:YES];
	connectingAlert = nil;

	if (stream == istream && event == NSStreamEventHasBytesAvailable) {
		NSMutableString *data = [NSMutableString string];
		while ([istream hasBytesAvailable]) {
			char buffer[1024];
			buffer[[istream read:(uint8_t *)buffer maxLength:(sizeof(buffer) - 1)]] = '\0';
			[data appendFormat:@"%s", buffer];
		}
		
		NSScanner *scanner = [NSScanner scannerWithString:data];
		while (![scanner isAtEnd]) {
			NSString *newMessage;
			[scanner scanUpToCharactersFromSet:[NSCharacterSet characterSetWithCharactersInString:@"#"] intoString:&newMessage];
		
			[message appendString:newMessage];
		
			if (![scanner isAtEnd]) {
				[scanner setScanLocation:([scanner scanLocation] + 1)];
			}
			else {
				return;
			}
		
			if ([message hasPrefix:@"MythFrontend Network Control"]) {
				[message setString:@""];
				continue;
			}
		
			[stateLock lock];
			if ([recvQueue count]) {
				NSDictionary *command = [[recvQueue objectAtIndex:0] retain];
				[recvQueue removeObjectAtIndex:0];
				if ([recvQueue count] <= 0) {
					[toSend setString:@""];
				}
				[stateLock unlock];
				
				if ([command objectForKey:@"Target"]) {
					[[command objectForKey:@"Target"] performSelector:((SelectorObject *)[command objectForKey:@"Selector"]).selector withObject:message];
					waitingReceive = false;
					[self processWrites];
				}
				
				[command release];
			}
			else {
				[stateLock unlock];
			}
			
			[message setString:@""];
		}
	}
	else if (stream == ostream && event == NSStreamEventHasSpaceAvailable) {
		[stateLock lock];
		if ([toSend length]) {
			NSInteger ret = [ostream write:(const uint8_t *)[toSend UTF8String] maxLength:[toSend length]];
			if (ret < 0) {
				[toSend setString:@""];
				[[[[UIAlertView alloc] initWithTitle:@"Remote Connection Error" message:@"The remote control connection to the Mythtv frontend has received an error." delegate:self cancelButtonTitle:@"Reconnect" otherButtonTitles:@"Frontend...", nil] autorelease] show];
				[istream close];
				[ostream close];
				for (NSDictionary *command in sendQueue) {
					[[command objectForKey:@"Target"] performSelector:((SelectorObject *)[command objectForKey:@"Selector"]).selector withObject:nil];
				}
				[sendQueue release];
				sendQueue = [[NSMutableArray alloc] init];
				for (NSDictionary *command in recvQueue) {
					[[command objectForKey:@"Target"] performSelector:((SelectorObject *)[command objectForKey:@"Selector"]).selector withObject:nil];
				}
				[recvQueue release];
				recvQueue = [[NSMutableArray alloc] init];
			}
			else {
				NSRange range;
				range.location = 0;
				range.length = ret;
				[toSend deleteCharactersInRange:range];
				if ([[recvQueue objectAtIndex:0] objectForKey:@"Target"] && ![toSend length]) {
					waitingReceive = true;
				}
			}
		}
		[stateLock unlock];
	}
	else if (event == NSStreamEventErrorOccurred) {
		[istream close];
		[ostream close];
		[stateLock lock];
		for (NSDictionary *command in sendQueue) {
			[[command objectForKey:@"Target"] performSelector:((SelectorObject *)[command objectForKey:@"Selector"]).selector withObject:nil];
		}
		[sendQueue release];
		sendQueue = [[NSMutableArray alloc] init];
		for (NSDictionary *command in recvQueue) {
			[[command objectForKey:@"Target"] performSelector:((SelectorObject *)[command objectForKey:@"Selector"]).selector withObject:nil];
		}
		[recvQueue release];
		recvQueue = [[NSMutableArray alloc] init];
		[toSend setString:@""];
		[stateLock unlock];
		
		// Don't prompt on errors where messages are in flight on closed connections as they are redundant to previous errors
		if (!resuming && ([[stream streamError] code] != 57 || ![[[stream streamError] domain] isEqualToString:@"NSPOSIXErrorDomain"]) && !TAB_BAR_CONTROLLER.modalViewController) {
			[[[[UIAlertView alloc] initWithTitle:@"Remote Connection Error" message:@"The remote control connection to the Mythtv frontend has received an error." delegate:self cancelButtonTitle:@"Reconnect" otherButtonTitles:@"Frontend...", nil] autorelease] show];
		}
		else {
			resuming = NO;
			[self setup];
		}
	}
	else if (event == NSStreamEventEndEncountered) {
		if (!TAB_BAR_CONTROLLER.modalViewController) {
			[[[[UIAlertView alloc] initWithTitle:@"Remote Connection Closed" message:@"The remote control connection to the Mythtv frontend closed unexpectedly." delegate:self cancelButtonTitle:@"Reconnect" otherButtonTitles:@"Frontends...", nil] autorelease] show];
		}
		[istream close];
		[ostream close];
		[stateLock lock];
		for (NSDictionary *command in sendQueue) {
			[[command objectForKey:@"Target"] performSelector:((SelectorObject *)[command objectForKey:@"Selector"]).selector withObject:nil];
		}
		[sendQueue release];
		sendQueue = [[NSMutableArray alloc] init];
		for (NSDictionary *command in recvQueue) {
			[[command objectForKey:@"Target"] performSelector:((SelectorObject *)[command objectForKey:@"Selector"]).selector withObject:nil];
		}
		[recvQueue release];
		recvQueue = [[NSMutableArray alloc] init];
		[toSend setString:@""];
		[stateLock unlock];
	}
}

- (void)processWrites {
	[stateLock lock];
	if ([sendQueue count] && ![toSend length]) {
		[toSend setString:[[sendQueue objectAtIndex:0] objectForKey:@"Query"]];
		[recvQueue addObject:[sendQueue objectAtIndex:0]];
		[sendQueue removeObjectAtIndex:0];
		[stateLock unlock];
		
		if ([ostream hasSpaceAvailable]) {
			[self stream:ostream handleEvent:NSStreamEventHasSpaceAvailable];
		}
	}
	else {
		[stateLock unlock];
	}
}

- (void)queryLocationWithTarget:(id)target selector:(SEL)selector {
	SelectorObject *selectorObject = [[[SelectorObject alloc] initWithSelector:selector] autorelease];
	[stateLock lock];
	[sendQueue addObject:[NSDictionary dictionaryWithObjectsAndKeys:@"query location\n", @"Query", target, @"Target", selectorObject, @"Selector", nil]];
	[stateLock unlock];
	[self processWrites];
}

- (void)sendCommand:(NSString *)command {
	NSString *fullCommand = [NSString stringWithFormat:@"%@\n", command];
	[stateLock lock];
	[sendQueue addObject:[NSDictionary dictionaryWithObjectsAndKeys:fullCommand, @"Query", nil]];
	[stateLock unlock];
	[self processWrites];
}

- (void)dealloc {
	[istream close];
	[ostream close];
	[istream removeFromRunLoop:[NSRunLoop mainRunLoop] forMode:NSDefaultRunLoopMode];
	[ostream removeFromRunLoop:[NSRunLoop mainRunLoop] forMode:NSDefaultRunLoopMode];
	[istream release];
	[ostream release];
	[stateLock lock];
	for (NSDictionary *command in sendQueue) {
		[[command objectForKey:@"Target"] performSelector:((SelectorObject *)[command objectForKey:@"Selector"]).selector withObject:nil];
	}
	[sendQueue release];
	for (NSDictionary *command in recvQueue) {
		[[command objectForKey:@"Target"] performSelector:((SelectorObject *)[command objectForKey:@"Selector"]).selector withObject:nil];
	}
	[recvQueue release];
	[toSend release];
	[stateLock unlock];
	[stateLock release];
	[message release];
	[netDownAlert dismissWithClickedButtonIndex:0 animated:NO];
	[netDownAlert release];
	[connectingAlert dismissWithClickedButtonIndex:0 animated:NO];
	[super dealloc];
}

// UIAlertViewDelegate

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {
	BackendSelect *backendSelect;
	
	if (alertView == connectingAlert) {
		connectingAlert = nil;
		backendSelect = [[[BackendSelect alloc] init] autorelease];
		UINavigationController *navigationController = [[[UINavigationController alloc] initWithRootViewController:backendSelect] autorelease];
		navigationController.navigationBar.barStyle = UIBarStyleBlackOpaque;
		[TAB_BAR_CONTROLLER presentModalViewController:navigationController animated:YES];
	}		
	else {
		switch (buttonIndex) {
			case 0:
				[self setup];
				break;
			
			case 1:
				backendSelect = [[[BackendSelect alloc] init] autorelease];
				UINavigationController *navigationController = [[[UINavigationController alloc] initWithRootViewController:backendSelect] autorelease];
				navigationController.navigationBar.barStyle = UIBarStyleBlackOpaque;
				[TAB_BAR_CONTROLLER presentModalViewController:navigationController animated:YES];
				break;
		}
	}
}

@end

@implementation SelectorObject

@synthesize selector;

- (id) initWithSelector:(SEL)newSelector {
	if (self = [super init]) {
		selector = newSelector;
	}
	return self;
}

@end