//
//  TimesView.m
//  MyMote
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//

#import <QuartzCore/CALayer.h>

#import "ProgramGuideData.h"
#import "ProgramGuideView.h"
#import "GuideResponse.h"
#import "TimeScrollView.h"
#import "TimesView.h"

@implementation TimesView

- (id)initWithProgramGuideData:(ProgramGuideData *)newData {
	if (self = [super init]) {
		self.frame = CGRectMake(0, 0, EPG_PROGRAMS_WIDTH, EPG_ROW_HEIGHT);
		self.clearsContextBeforeDrawing = NO;
		self.opaque = YES;

		data = [newData retain];
	}
	return self;
}

- (void)render {
	static NSDateFormatter *formatter = nil;
	if (!formatter) {
		formatter = [[NSDateFormatter alloc] init];
		formatter.dateFormat = @"h:mm a";
	}
	
	self.layer.contents = nil;
	
	NSDate *startDate = data.response.startDate;
	NSDateComponents *offset = [[NSDateComponents alloc] init];
	offset.second = 6 * 60 * 60 * self.bounds.size.width / EPG_PROGRAMS_WIDTH;
	NSDate *endDate = [[NSCalendar currentCalendar] dateByAddingComponents:offset toDate:data.response.startDate options:0];
	
	NSDateComponents *components = [[NSCalendar currentCalendar] components:(NSYearCalendarUnit | NSMonthCalendarUnit | NSDayCalendarUnit | NSHourCalendarUnit | NSMinuteCalendarUnit) fromDate:startDate];
	components.minute = (components.minute < 30 ? 0 : 30);
	startDate = [[NSCalendar currentCalendar] dateFromComponents:components];
	
	[offset release];
	offset = [[NSDateComponents alloc] init];
	offset.minute = 30;
	
	CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
	CGContextRef ctx = CGBitmapContextCreate(NULL, self.bounds.size.width * SCALE, self.bounds.size.height * SCALE, 8, self.bounds.size.width * SCALE * 4, colorSpace, kCGImageAlphaPremultipliedLast);
	
	CGContextScaleCTM(ctx, 1, -1);
	CGContextTranslateCTM(ctx, 0, -self.bounds.size.height * SCALE);
	
	while ([startDate compare:endDate] == NSOrderedAscending) {
		float guideStartOffset = [[NSCalendar currentCalendar] components:(NSMinuteCalendarUnit) fromDate:data.response.startDate toDate:startDate options:0].minute;
		CGRect timeRect = CGRectMake((guideStartOffset / data.response.guideMinuteLength * EPG_PROGRAMS_WIDTH + 1) * SCALE, SCALE, (30 / data.response.guideMinuteLength * EPG_PROGRAMS_WIDTH - 2) * SCALE, (EPG_ROW_HEIGHT - 2) * SCALE);
		
		CGContextAddRect(ctx, timeRect);
		CGContextSetRGBFillColor(ctx, 1, 1, 1, 1);
		CGContextFillPath(ctx);
	
		CGContextSetRGBFillColor(ctx, 0, 0, 0, 1);
		
		CGContextScaleCTM(ctx, 1, -1);
		CGContextTranslateCTM(ctx, 0, -timeRect.size.height - timeRect.origin.y * 2);
		
		[[formatter stringFromDate:startDate] drawCenteredInRect:timeRect withFont:[UIFont boldSystemFontOfSize:([UIFont systemFontSize] * SCALE)] context:ctx];
		
		CGContextTranslateCTM(ctx, 0, timeRect.size.height + timeRect.origin.y * 2);
		CGContextScaleCTM(ctx, 1, -1);
		
		startDate = [[NSCalendar currentCalendar] dateByAddingComponents:offset toDate:startDate options:0];
	}
	
	CGImageRef image = CGBitmapContextCreateImage(ctx);
	
	CGContextRelease(ctx);
	CGColorSpaceRelease(colorSpace);
	
	self.layer.contents = (id)image;
	if ([self.layer respondsToSelector:@selector(contentsScale)])
		self.layer.contentsScale = SCALE;
	CFRelease(image);

	[offset release];
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
	if (!((TimeScrollView *)self.superview).scrollEnabled) {
		return;
	}
	
	if ([touches count] == 1 && [self pointInside:[[touches anyObject] locationInView:self] withEvent:nil]) {
		[(ProgramGuideView *)self.superview.superview requestNewTime];
	}
}

- (void)dealloc {
	[data release];
	[super dealloc];
}

@end
