//
//  keyparser.m
//  MyMote
//
//  Copyright 2008 Chase Douglas
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  version 2 as published by the Free Software Foundation
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//
//  Use query, exported as CSV, as input: SELECT * FROM keybindings WHERE hostname = 'some_host';
//
//  Replace keys:
//  esc: escape
//  pgup: pageup
//  pgdown: pagedown

#include <Foundation/Foundation.h>

int main() {
	NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
	
	NSMutableDictionary *bindings = [[NSMutableDictionary alloc] init];
	
	BOOL firstLine = YES;
	while (!feof(stdin)) {
		char *characters;
		size_t length;
		
		characters = fgetln(stdin, &length);
		
		if (firstLine) {
			firstLine = NO;
			continue;
		}
		
		if (length <= 1) {
			continue;
		}
		
		NSString *line = [[NSString alloc] initWithBytesNoCopy:(characters + 1) length:(length - 3) encoding:NSASCIIStringEncoding freeWhenDone:NO];
		
		NSArray *fields = [line componentsSeparatedByString:@"\",\""];
		
		NSMutableDictionary *context;
		NSMutableDictionary *action;
		
		if (!(context = [bindings objectForKey:[fields objectAtIndex:0]])) {
			context = [[NSMutableDictionary alloc] init];
			[bindings setObject:context forKey:[fields objectAtIndex:0]];
			[context release];
		}
		
		if (!(action = [context objectForKey:[fields objectAtIndex:1]])) {
			action = [[NSMutableDictionary alloc] init];
			[context setObject:action forKey:[fields objectAtIndex:1]];
			[action release];
		}
		
		[action setObject:[fields objectAtIndex:2] forKey:@"Description"];
		
		if ([[fields objectAtIndex:3] length] > 0 && [[fields objectAtIndex:3] characterAtIndex:0] == ',') {
			[action setObject:@"," forKey:@"Key"];
		}
		else {
			NSArray *keys = [[fields objectAtIndex:3] componentsSeparatedByCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@","]];
			[action setObject:[[keys objectAtIndex:0] lowercaseString] forKey:@"Key"];
		}
		
		[line release];
	}
	
	NSData *plist = [NSPropertyListSerialization dataFromPropertyList:bindings format:NSPropertyListXMLFormat_v1_0 errorDescription:nil];
	
	[[NSFileHandle fileHandleWithStandardOutput] writeData:plist];
	
	[pool release];
}